
/*
*   EPT_SGTool.cpp - Library for the
*   Serial Graph Tool. 
*   created by Earth People Technology Inc. May 23, 2014
*   Released to public domain
*/

#include "Arduino.h"
#include "EPT_SGTool.h"
// include the SPI library
#include <SPI.h>

//EPT Serial Graph Tool 
#define HEADER_BYTE_1    0xC3
#define HEADER_BYTE_2    0xD2
#define HEADER_BYTE_3    0xE1
#define CMD_FRAME_END    0xB8
#define CMD_CHANNEL      0xA1


const int slaveSelectPin = 10;

void EPT_SGToolInit()
{
  pinMode (slaveSelectPin, OUTPUT);
  
  //Initialize SPI
  SPI.begin();

}

void EPT_SGToolFrameEnd()
{
         //take the SS pin low to select the chip;
         digitalWrite(slaveSelectPin, LOW);
         // send in the address and value via SPI:
         SPI.transfer(0x00);
         //send Header Byte 1
         SPI.transfer(HEADER_BYTE_1);
         //send Header Byte 2
         SPI.transfer(HEADER_BYTE_2);
         //send Header Byte 3
         SPI.transfer(HEADER_BYTE_3);
         //send Command Byte to update the Graph
         SPI.transfer(CMD_FRAME_END);
         //take the SS pin high to de-select the chip;
         digitalWrite(slaveSelectPin, HIGH);

}

void EPT_SGToolChannelSelect(int Channel)
{
  
         //take the SS pin low to select the chip;
         digitalWrite(slaveSelectPin, LOW);
         // send in the address and value via SPI:
         SPI.transfer(0x00);
         //send Header Byte 1
         SPI.transfer(HEADER_BYTE_1);
         //send Header Byte 2
         SPI.transfer(HEADER_BYTE_2);
         //send Header Byte 3
         SPI.transfer(HEADER_BYTE_3);
         //send Command Byte to update the Graph
         SPI.transfer(CMD_CHANNEL + Channel);
         //take the SS pin high to de-select the chip;
         digitalWrite(slaveSelectPin, HIGH);

}

void EPT_SGToolWriteBlockData(int *WriteBuffer, int SizeOfBlock, int Channel, int DataSize)
{
         int *ptr;
     
         ptr = WriteBuffer;
         
         EPT_SGToolChannelSelect(Channel);
  
         digitalWrite(slaveSelectPin, LOW);
         // send in the address and value via SPI:
         SPI.transfer(0x00);

         // go through for loop and incrementally send a count to SPI bus
         for( int count = 0; count < SizeOfBlock; count++)
         {
             if(*ptr)
             {
               if(DataSize > 1)
               {
                  SPI.transfer((*ptr&0xff00)>>8);  
                  SPI.transfer(*ptr&0xff);
                  *ptr++;               
               }
               else
                SPI.transfer(*ptr++);
             }
         }

         //Set the Slave Select pin high
         digitalWrite(slaveSelectPin, HIGH);

}
