//-----------------------------------------------------------------------------
// Arduino_Fast_Oscilloscope.ino
//-----------------------------------------------------------------------------
// Copyright 2012 Cristiano Lino Fontana
//
// This file is part of Girino.
//
//	Girino is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	Girino is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with Girino.  If not, see <http://www.gnu.org/licenses/>.
//
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

#include "Arduino_Fast_Oscilloscope.h"

//-----------------------------------------------------------------------------
// Global Constants
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------

volatile  boolean wait;
         uint16_t waitDuration;
volatile uint16_t stopIndex;
volatile uint16_t TriggerCount;
volatile uint8_t  ADCTempCount;
volatile uint16_t ADCCounter;
volatile  uint8_t ADCBuffer[ADCBUFFERSIZE];
volatile  boolean freeze;
volatile  boolean update;

          uint8_t prescaler;
          uint8_t triggerEvent;
          uint8_t threshold;

         char commandBuffer[COMBUFFERSIZE+1];
             
         uint16_t DataBufferSize;
         
         uint8_t WriteEnable;
         
         boolean OneTrigger;

//-----------------------------------------------------------------------------
// Main routines
//-----------------------------------------------------------------------------
//
// The setup function initializes registers.
//
void setup (void) {		// Setup of the microcontroller
	// Open serial port with a baud rate of BAUDRATE b/s
	Serial.begin(115200);

	// Clear buffers
	memset( (void *)ADCBuffer, 0, sizeof(ADCBuffer) );
	memset( (void *)commandBuffer, 0, sizeof(commandBuffer) );
	ADCCounter = 0;
	wait = false;
	//waitDuration = ADCBUFFERSIZE - 32;
	waitDuration = 450;
        ADCTempCount = 0;
	stopIndex = -1;
	freeze = false;
        update = false;

	prescaler = 128;
	triggerEvent = 3;

	threshold = 127;

        DataBufferSize = ADCBUFFERSIZE;

	// Activate interrupts
	//sei();
        interrupts(); 

	initPins();
	initADC();//252
	initAnalogComparator();

	//Serial.println("Girino ready");
	//printStatus();

        //Set the Analog pins 6, 5, 4, 2, 1 to outputs
        REG_PIOC_PER = B00111110;
        //REG_PIOC_0ER = B00111110;
        REG_PIOC_OWER = B11111111;
        REG_PIOC_ODSR &= B00000000;
        
        //Set the Port B pins 5-0 as outputs
        //REG_PIOB_PER = 0xffffffff;
        //REG_PIOB_0ER = 0x00000000;
        //REG_PIOB_OWER = 0xffffffff;
        //REG_PIOB_ODSR &= 0x00000000;
        pinMode(13, OUTPUT);
        //digitalWrite(13, LOW);
        
        REG_PIOD_PER = B00000000;
        //REG_PIOD_0ER = B00000000;
        REG_PIOD_OWER = B0000000;
        REG_PIOD_ODSR &= B00000000;
        
        OneTrigger = true;
        
        Serial.println("EPT Fast Oscilloscope Ver 1.012\n");
}

void loop (void) {
  
  delay(50);
  REG_PIOB_ODSR = 1<<27;

  delay(50);
  REG_PIOB_ODSR = 0<<27;

	// If freeze flag is set, then it is time to send the buffer to the serial port
	if ( freeze )
	{
		//dshow("# Frozen");

		// Send buffer through serial port in the right order
		EPT_USB_Write((uint8_t*)ADCBuffer, DataBufferSize);

		// Turn off errorPin
		//digitalWrite( errorPin, LOW );
		//cbi(PORTB,PORTB5);

		wait = false;
		freeze = false;

		// Clear buffer
		memset( (void *)ADCBuffer, 0, sizeof(ADCBuffer) );
                ADCCounter = 0;
                stopIndex = 0;
                TriggerCount = 0;


		startADC();
		// Let the ADC fill the buffer a little bit
		delay(100);
		startAnalogComparator();

	}

        if(update)
        {
 	   ADCBuffer[ADCCounter] = REG_ADC_CDR;

           if(ADCCounter < ADCBUFFERSIZE)
           {
              ADCCounter++; 
           }
           else
           {
              ADCCounter = 0;
           }
        
             update = false;
        }

	if ( Serial.available() > 0 ) {
		// Read the incoming byte
		char theChar = Serial.read();
			// Parse character
		switch (theChar) {
			case 's':			// 's' for starting ADC conversions
				Serial.println("ADC conversions started\n");

				// Clear buffer
				memset( (void *)ADCBuffer, 0, sizeof(ADCBuffer) );

				startADC();
				// Let the ADC fill the buffer a little bit
				delay(1);
				startAnalogComparator();
				break;
			case 'S':			// 'S' for stopping ADC conversions
				Serial.println("ADC conversions stopped\n");
				stopAnalogComparator();
				stopADC();
				break;
			case 'p':			// 'p' for new prescaler setting
			case 'P': {
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
                                //commandBuffer = "32";
                                //commandBuffer[1] = '2';
				//uint8_t newP = atoi( "64"/*commandBuffer*/ );
				uint8_t newP = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting the EPT prescaler to: ");
				Serial.println(newP);

				prescaler = newP;
				setADCPrescaler(newP);
				//setADCPrescaler(32);
				}
				break;

			case 'r':			// 'r' for new voltage reference setting
			case 'R': {
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
				uint8_t newR = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting voltage reference to: ");
				Serial.println(newR);

				setVoltageReference(newR);
				}
				break;

			case 'e':			// 'e' for new trigger event setting
			case 'E': {
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
				uint8_t newE = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting trigger event to: ");
				Serial.println(newE);

				triggerEvent = newE;
				setTriggerEvent(newE);
				}
				break;

			case 'w':			// 'w' for new wait setting
			case 'W': {
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
				uint8_t newW = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting waitDuration to: ");
				Serial.println(newW);

				waitDuration = newW;
				}
				break;

			case 't':			// 'w' for new threshold setting
			case 'T': {
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
				uint8_t newT = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting threshold to: ");
				Serial.println(newT);

				threshold = newT;
				analogWrite( thresholdPin, threshold );
				}
				break;

			case 'd':			// 'd' for display status
			case 'D':
				printStatus();
				break;

			default:
				// Display error message
				Serial.print("ERROR: Command not found, it was: ");
				Serial.println(theChar);
				error();
		}
	}
}
