﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Microsoft.Win32;

namespace EPT_Serial_Graph_Tool
{
    public partial class EPT_Serial_Graph_Tool : Form
    {
        int[] StorageArray = new int[50000];

        public EPT_Serial_Graph_Tool()
        {
            InitializeComponent();

            InitializeStorageArray();

            // reduce flicker

            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.DoubleBuffer, true);



        }

        //Parameters
        //Command Byte Detection Header Bytes
        public const byte HEADER_BYTE_1 = 0xC3;
        public const byte HEADER_BYTE_2 = 0xD2;
        public const byte HEADER_BYTE_3 = 0xE1;

        //Parameters for Voltage Reference Buttons
        public const int SELECT_VREF1 = 1;
        public const int SELECT_VREF2 = 2;
        public const int SELECT_VREF3 = 3;

        //Parameters for Voltage Reference Buttons
        public const int SELECT_TRIG1 = 1;
        public const int SELECT_TRIG2 = 2;
        public const int SELECT_TRIG3 = 3;

        //Parameters for the Fifo hold off
        public const int REFRESH_TIMEOUT = 150;
        public const int FIFO_HOLD_TIME = 1200;

        //Buffer to hold bytes transferred from the EPT Serial Graph Tool
        public int[][] ScopeBuffer = new int[8] [];
        public uint[] EPTReceiveIndex = new uint[8];
        public int OuterDimension = 0;

        // put the a/d readings into repeat mode for 450 readings
        int n = 0;

        //ScopeBuffer Channel Select
        public int ScopeBufferChannelSelect = 0;

        //Flag to indicate if the ScopeBuffer has collected the first
        //450 data points
        public bool EPTReceiveIndexInitFlag = false;

        //ScopeBuffer index for the graph section
        public uint ScopeIndex = 0;

        //Offset value to change the Horizontal position
        public int IndexOffset=0;

        //Flag to prevent BlockOutReceive or TriggerOutRecieve from writing
        // to the ScopeBuffer or initiating RefreshScope() while RefreshScope()
        //is in process
        public bool RefreshBusy = false;

        private bool StorageOn = false;

        //Prescalar value
        public char[] PreScalarValue = new char[10];

        //Voltage Reference value
        public char VoltageReference = '1';


        //Voltage Reference value
        public char TriggerEvent = '1';

        //FifoHold register
        public bool FifoHold = false;
        public bool SetFifoHoldEnable = false;

        //Prescalar value
        public char[] TriggerThreshold = new char[10];

        // This thread is used to demonstrate both thread-safe and
        // unsafe ways to call a Windows Forms control.
        private Thread DisplayThread = null;

        //Serial Port 1 received byte array
        byte[] ReceivedByteArray;

        // Local Serial Port parameters From Properties Page
        public static string PortName = "COM1";
        public static string BaudRate = "2400";
        public static string DataBits = "8";
        public static string vParity = "None";
        public static string StopBits = "1";
        public static string pHandshake = "None";

        //ScopeOn register to allow graph to be turned on/off
        bool ScopeOn = false;

        //Tel-D Command States for the Receive
        public const int SERDEV_STATE_IDLE     = 1;
        public const int SERDEV_STATE_HDR_2    = 2;
        public const int SERDEV_STATE_HDR_3    = 3;
        public const int SERDEV_STATE_CMD      = 4;

        //Serial Device Command State
        public int SerialDeviceCommandState = SERDEV_STATE_IDLE;

        //Serial Devie byte array to hold the payload
        public byte[] SerDev_Payload;

        //Command Message Array
        public byte[] CommandMessageArray = new byte[10];
        public int CommandIndex = 0;

        //Sting array to hold the Port names that match the VID = "403 PID = "6001"
        public string[] MatchingComPortList = new string[10];

        //Selected Channel control, used to get the value from the trackbars
        //and place them into the channel values
        public int ChannelSelectControl = 0;

        //Array of active channels, used to determine if a channel
        //memory block has been set up
        public bool[] ActiveChannels = new bool[8];

        //Registers to Determine if the incoming data is two bytes. Determine
        //whether it is the upper byte or lower byte.
        public bool TwoByteData = false;
        public bool UpperByte = true;

        //Scaling to be used on the incoming data
        public int BitScale = 256;

        //StreamWrite File
        StreamWriter writer;

        //Save File string
        public string Saved_File;
        public string previousPath = "";

        //Save Data To File Flag
        public bool SaveDataToFile = false;
        //Voltage Scale registers for each channel
        //Voltage Scale Factor
        public int VoltageScaleValue = 5;
        public int VScaleChannel_1 = 5;
        public int VScaleChannel_2 = 5;
        public int VScaleChannel_3 = 5;
        public int VScaleChannel_4 = 5;
        public int VScaleChannel_5 = 5;
        public int VScaleChannel_6 = 5;
        public int VScaleChannel_7 = 5;
        public int VScaleChannel_8 = 5;

        //Vertical Scale registers for each channel
        public int VerticalScaleValue = 0;
        public int VerticalScale_1 = 0;
        public int VerticalScale_2 = 0;
        public int VerticalScale_3 = 0;
        public int VerticalScale_4 = 0;
        public int VerticalScale_5 = 0;
        public int VerticalScale_6 = 0;
        public int VerticalScale_7 = 0;
        public int VerticalScale_8 = 0;

        //Horizontal Position registers
        public int HorizontalScaleValue = 0;
        public int HorizontalScale_1 = 0;
        public int HorizontalScale_2 = 0;
        public int HorizontalScale_3 = 0;
        public int HorizontalScale_4 = 0;
        public int HorizontalScale_5 = 0;
        public int HorizontalScale_6 = 0;
        public int HorizontalScale_7 = 0;
        public int HorizontalScale_8 = 0;

        //Time Scale registers
        public int TimeScaleValue = 1;
        public int TimeScale_1 = 1;
        public int TimeScale_2 = 1;
        public int TimeScale_3 = 1;
        public int TimeScale_4 = 1;
        public int TimeScale_5 = 1;
        public int TimeScale_6 = 1;
        public int TimeScale_7 = 1;
        public int TimeScale_8 = 1;

        //Horizontal Fast Shift Registers
        public int HorizontalFastShift = 0;
        public int HorizontalFastShift_1 = 0;
        public int HorizontalFastShift_2 = 0;
        public int HorizontalFastShift_3 = 0;
        public int HorizontalFastShift_4 = 0;
        public int HorizontalFastShift_5 = 0;
        public int HorizontalFastShift_6 = 0;
        public int HorizontalFastShift_7 = 0;
        public int HorizontalFastShift_8 = 0;

        //Labels to add
        Label Channel_1_VerticalLabel = null;
        Label Channel_2_VerticalLabel = null;
        Label Channel_3_VerticalLabel = null;
        Label Channel_4_VerticalLabel = null;
        Label Channel_5_VerticalLabel = null;
        Label Channel_6_VerticalLabel = null;
        Label Channel_7_VerticalLabel = null;
        Label Channel_8_VerticalLabel = null;

        // Main object loader
        private void EPT_Oscilloscope_Load(object sender, System.EventArgs e)
        {

            //String buffer
            String PortText = "";

            //Index registers
            int Index = 0;

            // Call the List Devices function
            List<string> names = ComPortNames("0403", "6015");

            // Get a list of serial port names.
            string[] ports;
            ports = SerialPort.GetPortNames();


            if (names.Count > 0)
            {
                foreach (String port in ports)
                {
                    //Compare port name with the found VID/PID 
                    //combinations. Add them to Matching port list
                    //and comboDevList
                    if (names.Contains(port))
                    {
                        MatchingComPortList[Index] = port;

                        PortText = "Serial Graph Tool " + Index++;
                        cmbDevList.Items.Add(PortText);
                    }
                }
            }
            else
                MessageBox.Show("No Serial Graph Tools found!");

            //Set the Voltage Scale
            lblVoltScale.Text = "Unity Scaling";

            //Set Label to blank
            lblDeviceConnected.Text = " ";

            //Set the Horizontal Time Base label
            rdo8BitSelect.Checked = true;

        }


        private void SetButtonEnables()
        {
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            btnStart.Enabled = true;
            btnStop.Enabled = false;
        }

        private void cmbDevList_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Get Port name from Combo Box
            PortName = MatchingComPortList[cmbDevList.SelectedIndex];
        }

        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            OpenSerialPort1();

            lblDeviceConnected.Text = "Device Connected";
        }

        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            CloseSerialPort1();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            // put something on pin 0
            ScopeOn = true;

            btnStart.Enabled = false;
            btnStop.Enabled = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            // put something on pin 0
            ScopeOn = false;

            btnStart.Enabled = true;
            btnStop.Enabled = false;

            Invalidate();

        }

        //This is an alias to the Properties menu item under File
        //private void btnProperties_Click(object sender, EventArgs e)
        //{
            //ShowChildForm();
        //}

        //Open the Properties Window

        public void CloseSerialPort1()
        {
            //Check if serialPort1 is open, then close the serial port and window
            if (serialPort1.IsOpen)
            {
                serialPort1.Close();
            }
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            //textBox1.ReadOnly = true;

        }

        public bool OpenSerialPort1()
        {
            try
            {
                //Set the serial port parameters
                serialPort1.PortName = PortName;
                serialPort1.BaudRate = Convert.ToInt32(BaudRate);
                serialPort1.Parity = (Parity)Enum.Parse(typeof(Parity), vParity);
                serialPort1.DataBits = Convert.ToInt16(DataBits);
                serialPort1.StopBits = (StopBits)Enum.Parse(typeof(StopBits), StopBits);
                serialPort1.Handshake = (Handshake)Enum.Parse(typeof(Handshake), pHandshake);

                if (!serialPort1.IsOpen)
                {
                    serialPort1.Open();
                    btnOpenDevice.Enabled = false;
                    btnCloseDevice.Enabled = true;
                    //textBox1.ReadOnly = false;
                    return true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            return false;
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            Invalidate();
        }

        void InitializeStorageArray()
        {
            for (int i = 0; i < StorageArray.Length; i++)
            {
                StorageArray[i] = 0;
            }

            //Initialize the first channel of the Serial Graph Tool
            ActiveChannels[0] = true; 
            ScopeBuffer[0] = new int[50000];
            cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1));
            Channel_1_VerticalLabel = new Label();
            Channel_1_VerticalLabel.ForeColor = Color.DarkGreen; 
            Channel_1_VerticalLabel.Location = new Point(450, 250);
            Channel_1_VerticalLabel.Font = new Font("Arial", 8);
            Channel_1_VerticalLabel.Text = "--Ch 1";
            this.Controls.Add(Channel_1_VerticalLabel);
        }

        Random r = new Random();

        private void RefreshScope(Graphics g)
        {
            int xmax = 450; // number of sample points
            int ymax = 256;  // maximum y value
            int xlast = -1;
            int ylast = 0;
            int xDraw = 0;
            int yDraw = 0;

            int nToggleColor = 0;
            int TimeScaleIn = 0;

            // put the a/d readings into repeat mode for 450 readings
            //int n = 0;

            RefreshBusy = true;

            try
            {
                for (int OuterDimension = 0; OuterDimension < 8; OuterDimension++)
                {
                    xlast = -1;
                    ylast = 0;
                    xDraw = 0;
                    yDraw = 0;
                    TimeScaleIn = 0;
                    //n = 0;

                    switch (OuterDimension)
                    {
                        case 0:
                            VoltageScaleValue = VScaleChannel_1;
                            VerticalScaleValue = VerticalScale_1;
                            HorizontalScaleValue = HorizontalScale_1;
                            TimeScaleValue = TimeScale_1;
                            IndexOffset = HorizontalScale_1 + HorizontalFastShift_1;
                            break;
                        case 1:
                            VoltageScaleValue = VScaleChannel_2;
                            VerticalScaleValue = VerticalScale_2;
                            HorizontalScaleValue = HorizontalScale_2;
                            TimeScaleValue = TimeScale_2;
                            IndexOffset = HorizontalScale_2 + HorizontalFastShift_2;
                            break;
                        case 2:
                            VoltageScaleValue = VScaleChannel_3;
                            VerticalScaleValue = VerticalScale_3;
                            HorizontalScaleValue = HorizontalScale_3;
                            TimeScaleValue = TimeScale_3;
                            IndexOffset = HorizontalScale_3 + HorizontalFastShift_3;
                            break;
                        case 3:
                            VoltageScaleValue = VScaleChannel_4;
                            VerticalScaleValue = VerticalScale_4;
                            HorizontalScaleValue = HorizontalScale_4;
                            TimeScaleValue = TimeScale_4;
                            IndexOffset = HorizontalScale_4 + HorizontalFastShift_4;
                            break;
                        case 4:
                            VoltageScaleValue = VScaleChannel_5;
                            VerticalScaleValue = VerticalScale_5;
                            HorizontalScaleValue = HorizontalScale_5;
                            TimeScaleValue = TimeScale_5;
                            IndexOffset = HorizontalScale_5 + HorizontalFastShift_5;
                            break;
                        case 5:
                            VoltageScaleValue = VScaleChannel_6;
                            VerticalScaleValue = VerticalScale_6;
                            HorizontalScaleValue = HorizontalScale_6;
                            TimeScaleValue = TimeScale_6;
                            IndexOffset = HorizontalScale_6 + HorizontalFastShift_6;
                            break;
                        case 6:
                            VoltageScaleValue = VScaleChannel_7;
                            VerticalScaleValue = VerticalScale_7;
                            HorizontalScaleValue = HorizontalScale_7;
                            TimeScaleValue = TimeScale_7;
                            IndexOffset = HorizontalScale_7 + HorizontalFastShift_7;
                            break;
                        case 7:
                            VoltageScaleValue = VScaleChannel_8;
                            VerticalScaleValue = VerticalScale_8;
                            HorizontalScaleValue = HorizontalScale_8;
                            TimeScaleValue = TimeScale_8;
                            IndexOffset = HorizontalScale_8 + HorizontalFastShift_8;
                            break;
                        default:
                            break;
                    }

                    ScopeIndex = GetScopeBufferIndex(OuterDimension, IndexOffset);
                    //if (ScopeBuffer[OuterDimension] != null)
                    if (ActiveChannels[OuterDimension])
                    {
                        // read each value from the buffer and plot the sample on the scope		
                        for (int xpos = 0; xpos < xmax; xpos++)
                        {
                            if (StorageOn)
                                n = StorageArray[xpos];
                            else
                                StorageArray[xpos] = n;

                            // n between 0 and 1023
                            //System.Single y = 0;
                            //System.Single volt = 0;
                            int yint = 0;

                            //if (SaveDataToFile)
                            //    writer.Write("Channel {0} Value {1:0} ", OuterDimension, n);


                            //Scale the voltage according to the voltage scale control
                            if (TwoByteData)
                            {
                                double Data;
                                Data = (double)((n * 450 )/BitScale);

                                //n = (int)Math.Ceiling(Data);
                                n = (int)Data;

                                ymax = 256;
                            }

                            //if (SaveDataToFile)
                            //    writer.Write("Channel {0} Pre-Calc Value {1:0} \n",OuterDimension, n);

                            if (VoltageScaleValue >= 5)
                            {
                                yint = (int)(n / (VoltageScaleValue - 4));
                            }
                            else
                            {
                                yint = (int)(n * (6 - VoltageScaleValue));
                            }

                            // offset the voltage based on the y position dial
                            yint -= -(VerticalScaleValue);

                            // place the y reading into the correct coordinate system
                            //    by subtracting from the maximum value
                            yDraw = ymax - yint;

                            // don't draw the first sample point, use it as
                            // a starting sample point
                            if (xlast != -1)
                            {
                                // scale the x-value based on the TimeScale control
                                // translate the x-position by the x-position control
                                //xDraw = xpos;
                                //xDraw = xpos * (int)this.TimeScaleControl.Value + ((int)trkTimeScaleControl.Value * (int)TimeScaleControl.Value * 45);
                                //xDraw = xpos * (int)this.trkTimeScale.Value + (HorizontalScaleValue * (int)trkTimeScale.Value * 45);
                                //xDraw = xpos * TimeScaleValue + (HorizontalScaleValue * TimeScaleValue * 5);
                                if (TimeScaleValue == 1)
                                    xDraw = xpos * TimeScaleValue;
                                else if (TimeScaleValue > 1)
                                    xDraw = xpos * (TimeScaleValue * 2);
                                else 
                                    xDraw = xpos;

                                // Draw the scaled sample point by connecting
                                // the previous scaled sample point to the current
                                // scaled sample point
                                if (xDraw < kScopeWidth)
                                {
                                    /*if ((nToggleColor % 2 == 0) || (ShowSamplesOn == false))
                                        g.DrawLine(Pens.LawnGreen, xlast, ylast, xDraw, yDraw);
                                    else
                                        g.DrawLine(Pens.Red, xlast, ylast, xDraw, yint);
                                    */
                                    switch (OuterDimension)
                                    {
                                        case 0:
                                            g.DrawLine(Pens.LawnGreen, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 1:
                                            g.DrawLine(Pens.Red, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 2:
                                            g.DrawLine(Pens.Blue, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 3:
                                            g.DrawLine(Pens.Orange, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 4:
                                            g.DrawLine(Pens.AntiqueWhite, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 5:
                                            g.DrawLine(Pens.DarkOliveGreen, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 6:
                                            g.DrawLine(Pens.DimGray, xlast, ylast, xDraw, yDraw);
                                            break;
                                        case 7:
                                            g.DrawLine(Pens.LightCyan, xlast, ylast, xDraw, yDraw);
                                            break;
                                        default:
                                            break;
                                    }
                                }
                            }

                            // remember the current sample point to allow for connecting
                            // the next sample point
                            xlast = xDraw;
                            ylast = yDraw;

                            nToggleColor++;

                            // read the next a/d value from the buffer
                            if (xpos <= xmax)
                            {
                                /*
                                if (ScopeIndex < 49999)
                                    n = ScopeBuffer[OuterDimension][ScopeIndex++];
                                else
                                {
                                    ScopeIndex = 0;
                                    n = ScopeBuffer[OuterDimension][ScopeIndex++];
                                }
                                */
                                
                                if (ScopeIndex > 49998)
                                    ScopeIndex = 0;

                                if (TimeScaleValue < 0)
                                {
                                    if (ScopeIndex + (TimeScaleIn * Math.Abs(TimeScaleValue)) < 49999)
                                        n = ScopeBuffer[OuterDimension][ScopeIndex++ + (TimeScaleIn++ * Math.Abs(TimeScaleValue))];
                                    else
                                        n = ScopeBuffer[OuterDimension][(TimeScaleIn++ * Math.Abs(TimeScaleValue))];
                                }
                                else
                                    n = ScopeBuffer[OuterDimension][ScopeIndex++];
                            }
                        }
                    }
                }
                //RefreshBusy is reset to allow the samples to start filling
                //the ScopeBuffer
                RefreshBusy = false;

            }
            catch (Exception ex)
            {
                MessageBox.Show("RefreshScope Error On Channel: " + OuterDimension + "    " + ex);
            }
        }

        const int kScopeWidth = 450;
        const int kScopeHeight = 450;
        Rectangle scopeRect = new Rectangle(0, 0, kScopeWidth, kScopeHeight);

        private void DrawGrid(Graphics g)
        {
            // Draw the Whole screen black
            g.FillRectangle(Brushes.Black, scopeRect);

            const int kpadding = 0;
            const int tickheight = 4;
            const int tickheightLarge = 8;
            // draw the Grid Lines
            for (int i = 0; i < 450; i += 50)
            {
                // draw horizontal line
                g.DrawLine(Pens.DarkGray, 0, i + kpadding, kScopeWidth, i + kpadding);

                // draw vertical line
                g.DrawLine(Pens.DarkGray, i + kpadding, 0, i + kpadding, kScopeHeight);

                // draw ticks on each line
                if (i == 200)
                {
                    for (int j = 0; j < kScopeWidth; j += 5)
                    {
                        if (j % 25 != 0)
                        {
                            g.DrawLine(Pens.White, j, (i - tickheight / 2),
                                j, (i + tickheight / 2));

                            g.DrawLine(Pens.White, (i - tickheight / 2), j,
                                (i + tickheight / 2), j);
                        }
                        else
                        {
                            g.DrawLine(Pens.White, j, (i - tickheightLarge / 2) + kpadding,
                                j, (i + tickheightLarge / 2) + kpadding);

                            g.DrawLine(Pens.White, (i - tickheightLarge / 2), j,
                                (i + tickheightLarge / 2), j);
                        }
                    }
                }
            }


            // Draw Scope from a to d
            //if (ScopeOn)
                this.RefreshScope(g);
        }


        private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            try
            {
                // Draw the scope Grid
                DrawGrid(g);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Form1_Paint Error:" + ex);
            }

        }


        private bool ShowSamplesOn = false;
        private void ShowSampleCheck_CheckedChanged(object sender, System.EventArgs e)
        {
            ShowSamplesOn = false;// this.ShowSampleCheck.Checked;
        }



        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //The serialPort1 can recieve characters in 64 byte packets once every 125 microseconds. So,
            // this event must process the recieved characters in protected threads as fast as possible.
            if(ScopeOn)
               this.Invoke(new EventHandler(CallGetData));
        }



        //CallGetData is invoked to process the data in the serial port 1 buffer
        private void CallGetData(object sender, EventArgs e)
        {
            try
            {
                ReceivedByteArray = new byte[serialPort1.BytesToRead];
                serialPort1.Read(ReceivedByteArray, 0, serialPort1.BytesToRead);
                this.DisplayThread = new Thread(new ThreadStart(this.BLockOutReceive));
                this.DisplayThread.Start();
            }
               catch (Exception ex)
            {
                //The CallGetData() is a process that was invoked from the serialPort1 recieve 
                //event. So, we need to handle errors fast. If a failure occurs, just let 
                //the OS kill the thread.
            }
        }

        public int SerDev_DCommandReceiveDecode(byte SerDev_DReceiveByte)
        {
            //
            // Serial Device Command State Machine
            /*
                SERDEV_STATE_IDLE       = 1
                SERDEV_STATE_HDR_2      = 2
                SERDEV_STATE_HDR_3      = 3
                SERDEV_STATE_CMD        = 4
            */


            switch (SerialDeviceCommandState)
            {
                case SERDEV_STATE_IDLE:
                    if (SerDev_DReceiveByte == HEADER_BYTE_1)
                    {
                        CommandMessageArray[CommandIndex++] = SerDev_DReceiveByte;
                        SerialDeviceCommandState = SERDEV_STATE_HDR_2;
                    }
                    else
                    {
                        SerialDeviceCommandState = SERDEV_STATE_IDLE;
                        CommandIndex = 0;
                        return 0;
                    }
                    break;
                case SERDEV_STATE_HDR_2:
                    if (SerDev_DReceiveByte == HEADER_BYTE_2)
                    {
                        CommandMessageArray[CommandIndex++] = SerDev_DReceiveByte;
                        SerialDeviceCommandState = SERDEV_STATE_HDR_3;
                    }
                    else
                    {
                        SerialDeviceCommandState = SERDEV_STATE_IDLE;
                        CommandIndex = 0;
                        return -2;
                    }
                    break;
                case SERDEV_STATE_HDR_3:
                    if (SerDev_DReceiveByte == HEADER_BYTE_3)
                    {
                        CommandMessageArray[CommandIndex++] = SerDev_DReceiveByte;
                        SerialDeviceCommandState = SERDEV_STATE_CMD;
                    }
                    else
                    {
                        SerialDeviceCommandState = SERDEV_STATE_IDLE;
                        CommandIndex = 0;
                        return -3;
                    }
                    break;
                case SERDEV_STATE_CMD:
                    CommandMessageArray[CommandIndex] = SerDev_DReceiveByte;
                    SerialDeviceCommandState = SERDEV_STATE_IDLE;
                    CommandIndex = 0;
                    return -1;
                    break;
                default:
                    return 0;
                    break;
            }

            return -4;

        }


        public void BLockOutReceive()
        {
            try
            {
                    //Store the incoming bytes into the ScopeBuffer.
                    //Increment the EPTReceiveIndex. When it reaches 500
                    // Redraw the graph
                    foreach (byte b in ReceivedByteArray)
                    {
                        switch(SerDev_DCommandReceiveDecode(b))
                        {
                            case 0:
                                 //Valid Data with no Header Bytes in the stream. Store it.
                                if (TwoByteData)
                                {
                                    if (UpperByte)
                                    {
                                        ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b << 8;
                                        UpperByte = false;
                                    }
                                    else
                                    {
                                        ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)b;
                                        UpperByte = true;
                                    }
                                }
                                else
                                    ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b;

                                 break;
                            case -1:
                                 //Command state machine has detected a command. Call CommandParser
                                 CommandParser(CommandMessageArray[3]);
                                 break;
                            case -2:
                                 //Header Byte 1 was detected but the sequence was not complete. So, store the byte.
                                 if (TwoByteData)
                                 {
                                     if (UpperByte)
                                     {
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)CommandMessageArray[0] << 8;
                                         UpperByte = !UpperByte;
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)b;
                                         UpperByte = !UpperByte;
                                     }
                                     else
                                     {
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)CommandMessageArray[0];
                                         UpperByte = !UpperByte;
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b << 8;
                                         //ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)CommandMessageArray[0];
                                         UpperByte = !UpperByte;
                                     }
                                     //UpperByte = !UpperByte;
                                 }
                                 else
                                 {
                                     ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)CommandMessageArray[0];
                                     ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b;
                                 }
                                 break;
                            case -3:
                                 //Header Byte 1 followed by a two was detected but the sequence was not complete. So, store the CommandMessageArray.
                                 foreach (byte c in CommandMessageArray)
                                 {
                                     if (TwoByteData)
                                     {
                                         if (UpperByte)
                                         {
                                             ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)c << 8;
                                             UpperByte = !UpperByte;
                                         }
                                         else
                                         {
                                             ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)c;
                                             UpperByte = !UpperByte;
                                         }
                                     }
                                     else
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)c;
                                 }
                                 if (TwoByteData)
                                 {
                                     if (UpperByte)
                                     {
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b << 8;
                                         UpperByte = !UpperByte;
                                     }
                                     else
                                     {
                                         ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] |= (int)b;
                                         UpperByte = !UpperByte;
                                    }
                                 }
                                 else
                                     ScopeBuffer[ScopeBufferChannelSelect][IncrEPTReceiveIndex(ScopeBufferChannelSelect, TwoByteData, UpperByte)] = (int)b;
                                 break;
                            default:
                                 break;
                        }


                    }
            }
            catch (Exception ex)
            {
                MessageBox.Show("EPTReceiveIndex = " + EPTReceiveIndex + " EPTParseReceive Error: " + ex);
            }


        }

        public uint IncrEPTReceiveIndex(int SelectedChannel, bool MultiByteData, bool UpperLowerByte)
        {

            //Check the Index for a roll over condition, if below
            //rollover point, increment Index
            if (EPTReceiveIndex[SelectedChannel] > 49998)
                EPTReceiveIndex[SelectedChannel] = 0;

            //Check the Index to see if it has passed 450. If it has
            //not passed 450, the graph display will have to pad the 
            //Scope buffer data with zeros. This flag is used to communicate
            //if the Index has passed the initial 450.
            if (EPTReceiveIndex[SelectedChannel] > 450)
                EPTReceiveIndexInitFlag = true;

            //Check if the Multiple byte data structure selected, if so then determine
            //if it is the upper or lower byte.
            if(MultiByteData)
                if(UpperLowerByte)
                    return EPTReceiveIndex[SelectedChannel];
                else
                    return EPTReceiveIndex[SelectedChannel]++;
            else
                return EPTReceiveIndex[SelectedChannel]++;
        }

        public uint GetScopeBufferIndex(int SelectedChannel, int IndexOffset)
        {
            if (!EPTReceiveIndexInitFlag)
                return 0;
            else if (EPTReceiveIndex[SelectedChannel] < (451 + IndexOffset))
                return (uint)((EPTReceiveIndex[SelectedChannel] + 1) - (451 + IndexOffset)) + 49999;
            else
                return (uint)(EPTReceiveIndex[SelectedChannel] - 451 + IndexOffset);
        }

        public void CommandParser(byte CommandByte)
        {
            //Set the UpperByte register to true whenever a command is received.
            //This will ensure that if the data is multi byte, the first byte 
            //is always the upper byte.
            UpperByte = true;

            //Check which command was sent from Serial Graph Tool
            switch (CommandByte)
            {
                case 0xA1:
                    ScopeBufferChannelSelect = 0;
                    if (ScopeBuffer[0] == null)
                    {
                        ActiveChannels[0] = true;
                        ScopeBuffer[0] = new int[50000];
                        cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1));
                    }
                    break;
                case 0xA2:
                        ScopeBufferChannelSelect = 1;
                        if (ScopeBuffer[1] == null)
                        {
                            ActiveChannels[1] = true;
                            ScopeBuffer[1] = new int[50000];
                            this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                            this.Invoke(new MethodInvoker(delegate() { Channel_2_VerticalLabel = new Label();  }));
                            this.Invoke(new MethodInvoker(delegate() { Channel_2_VerticalLabel.ForeColor = Color.Red; }));
                            this.Invoke(new MethodInvoker(delegate() { Channel_2_VerticalLabel.Location = new Point(450, 250); }));
                            this.Invoke(new MethodInvoker(delegate() { Channel_2_VerticalLabel.Font = new Font("Arial", 8);  }));
                            this.Invoke(new MethodInvoker(delegate() { Channel_2_VerticalLabel.Text = "--Ch 2";  }));
                            this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_2_VerticalLabel);   }));
                        }
                    break;
                case 0xA3:
                    ScopeBufferChannelSelect = 2;
                    if (ScopeBuffer[2] == null)
                    {
                        ActiveChannels[2] = true;
                        ScopeBuffer[2] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_3_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_3_VerticalLabel.ForeColor = Color.Blue; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_3_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_3_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_3_VerticalLabel.Text = "--Ch 3"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_3_VerticalLabel); }));
                    }
                    break;
                case 0xA4:
                    ScopeBufferChannelSelect = 3;
                    if (ScopeBuffer[3] == null)
                    {
                        ActiveChannels[3] = true;
                        ScopeBuffer[3] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_4_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_4_VerticalLabel.ForeColor = Color.Orange; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_4_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_4_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_4_VerticalLabel.Text = "--Ch 4"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_4_VerticalLabel); }));
                    }
                    break;
                case 0xA5:
                    ScopeBufferChannelSelect = 4;
                    if (ScopeBuffer[4] == null)
                    {
                        ActiveChannels[4] = true;
                        ScopeBuffer[4] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_5_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_5_VerticalLabel.ForeColor = Color.Black; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_5_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_5_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_5_VerticalLabel.Text = "--Ch 5"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_5_VerticalLabel); }));
                    }
                    break;
                case 0xA6:
                    ScopeBufferChannelSelect = 5;
                    if (ScopeBuffer[5] == null)
                    {
                        ActiveChannels[5] = true;
                        ScopeBuffer[5] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_6_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_6_VerticalLabel.ForeColor = Color.DarkOliveGreen; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_6_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_6_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_6_VerticalLabel.Text = "--Ch 6"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_6_VerticalLabel); }));
                    }
                    break;
                case 0xA7:
                    ScopeBufferChannelSelect = 6;
                    if (ScopeBuffer[6] == null)
                    {
                        ActiveChannels[6] = true;
                        ScopeBuffer[6] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_7_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_7_VerticalLabel.ForeColor = Color.DimGray; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_7_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_7_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_7_VerticalLabel.Text = "--Ch 7"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_7_VerticalLabel); }));
                    }
                    break;
                case 0xA8:
                    ScopeBufferChannelSelect = 7;
                    if (ScopeBuffer[7] == null)
                    {
                        ActiveChannels[7] = true;
                        ScopeBuffer[7] = new int[50000];
                        this.Invoke(new MethodInvoker(delegate() { cmbChannelSelect.Items.Add((ScopeBufferChannelSelect + 1)); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_8_VerticalLabel = new Label(); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_8_VerticalLabel.ForeColor = Color.Black; }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_8_VerticalLabel.Location = new Point(450, 250); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_8_VerticalLabel.Font = new Font("Arial", 8); }));
                        this.Invoke(new MethodInvoker(delegate() { Channel_8_VerticalLabel.Text = "--Ch 8"; }));
                        this.Invoke(new MethodInvoker(delegate() { this.Controls.Add(Channel_8_VerticalLabel); }));
                    }
                    break;
                case 0xB8:
                    if (!RefreshBusy & ScopeOn)
                    {
                        Invalidate();
                        //EPTReceiveIndex = 0;
                    }
                    break;

                default:
                    break;
            }
        }
        List<string> ComPortNames(String VID, String PID)
        {
            String pattern = String.Format("^VID_{0}.PID_{1}", VID, PID);
            Regex _rx = new Regex(pattern, RegexOptions.IgnoreCase);
            List<string> comports = new List<string>();
            RegistryKey rk1 = Registry.LocalMachine;
            RegistryKey rk2 = rk1.OpenSubKey("SYSTEM\\CurrentControlSet\\Enum");
            foreach (String s3 in rk2.GetSubKeyNames())
            {
                RegistryKey rk3 = rk2.OpenSubKey(s3);
                foreach (String s in rk3.GetSubKeyNames())
                {
                    if (_rx.Match(s).Success)
                    {
                        RegistryKey rk4 = rk3.OpenSubKey(s);
                        foreach (String s2 in rk4.GetSubKeyNames())
                        {
                            RegistryKey rk5 = rk4.OpenSubKey(s2);
                            comports.Add((string)rk5.GetValue("Device Parameters"));
                            RegistryKey rk6 = rk5.OpenSubKey("Device Parameters");
                            comports.Add((string)rk6.GetValue("PortName"));
                        }
                    }
                }
            }
            return comports;
        }

        private void cmbChannelSelect_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Select the channel from the combo box selection
            ChannelSelectControl =  Convert.ToInt32(cmbChannelSelect.Text);

            //Set all of the Channel Indicators to Regular Font
            SetChannelFontToRegular();

            //Set all of the sliders to the previously selected channel positions
            //Set the Select Channel Indicator to Bold
            switch (ChannelSelectControl)
            {
                case 1:
                      trkVoltageScale.Value = VScaleChannel_1;
                      trkVerticalScaleControl.Value = VerticalScale_1;
                      trkHorizScaleControl.Value = HorizontalScale_1;
                      trkTimeScale.Value = TimeScale_1;
                      if (ActiveChannels[ChannelSelectControl - 1])
                      {
                          Channel_1_VerticalLabel.Font = new Font("Arial", 10);
                          Channel_1_VerticalLabel.Font = new Font(Channel_1_VerticalLabel.Font, FontStyle.Bold);
                          btnChannelOnOff.Text = "On";
                      }
                      else
                          btnChannelOnOff.Text = "Off";
                    break;
                case 2:
                    trkVoltageScale.Value = VScaleChannel_2;
                    trkVerticalScaleControl.Value = VerticalScale_2;
                    trkHorizScaleControl.Value = HorizontalScale_2;
                    trkTimeScale.Value = TimeScale_2;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_2_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_2_VerticalLabel.Font = new Font(Channel_2_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                case 3:
                    trkVoltageScale.Value = VScaleChannel_3;
                    trkVerticalScaleControl.Value = VerticalScale_3;
                    trkHorizScaleControl.Value = HorizontalScale_3;
                    trkTimeScale.Value = TimeScale_3;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_3_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_3_VerticalLabel.Font = new Font(Channel_3_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                case 4:
                    trkVoltageScale.Value = VScaleChannel_4;
                    trkVerticalScaleControl.Value = VerticalScale_4;
                    trkHorizScaleControl.Value = HorizontalScale_4;
                    trkTimeScale.Value = TimeScale_4;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_4_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_4_VerticalLabel.Font = new Font(Channel_4_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                case 5:
                    trkVoltageScale.Value = VScaleChannel_5;
                    trkVerticalScaleControl.Value = VerticalScale_5;
                    trkHorizScaleControl.Value = HorizontalScale_5;
                    trkTimeScale.Value = TimeScale_5;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_5_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_5_VerticalLabel.Font = new Font(Channel_5_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                   break;
                case 6:
                    trkVoltageScale.Value = VScaleChannel_6;
                    trkVerticalScaleControl.Value = VerticalScale_6;
                    trkHorizScaleControl.Value = HorizontalScale_6;
                    trkTimeScale.Value = TimeScale_6;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_6_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_6_VerticalLabel.Font = new Font(Channel_6_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                case 7:
                    trkVoltageScale.Value = VScaleChannel_7;
                    trkVerticalScaleControl.Value = VerticalScale_7;
                    trkHorizScaleControl.Value = HorizontalScale_7;
                    trkTimeScale.Value = TimeScale_7;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_7_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_7_VerticalLabel.Font = new Font(Channel_7_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                case 8:
                    trkVoltageScale.Value = VScaleChannel_8;
                    trkVerticalScaleControl.Value = VerticalScale_8;
                    trkHorizScaleControl.Value = HorizontalScale_8;
                    trkTimeScale.Value = TimeScale_8;
                    if (ActiveChannels[ChannelSelectControl - 1])
                    {
                        Channel_8_VerticalLabel.Font = new Font("Arial", 10);
                        Channel_8_VerticalLabel.Font = new Font(Channel_8_VerticalLabel.Font, FontStyle.Bold);
                        btnChannelOnOff.Text = "On";
                    }
                    else
                        btnChannelOnOff.Text = "Off";
                    break;
                default:
                    break;
            }

            //Set the Voltage Scale
            double VoltageScaleC = 0;

            if (trkVoltageScale.Value == 5)
            {
                VoltageScaleC = trkVoltageScale.Value - 4;
                lblVoltScale.Text = "Vertical Scale:\n 1";
            }
            else if (trkVoltageScale.Value > 5)
            {
                VoltageScaleC = trkVoltageScale.Value - 4;
                lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
            }
            else
            {
                VoltageScaleC = (double)(VScaleChannel_1 / 5.0);
                //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                lblVoltScale.Text = "Vertical Scale:\n * " + trkVoltageScale.Value;
            }
        }

        private void trkVoltageScale_Scroll(object sender, EventArgs e)
        {
            //Declare a register to display on the Voltage Scale label
            double VoltageScaleC=0;

            switch (ChannelSelectControl)
            {
                case 1:
                    VScaleChannel_1 = trkVoltageScale.Value;
                    if (VScaleChannel_1 == 5)
                    {
                        VoltageScaleC = VScaleChannel_1 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_1 > 5)
                    {
                        VoltageScaleC = VScaleChannel_1 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_1 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_1;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 2:
                    VScaleChannel_2 = trkVoltageScale.Value;
                    if(VScaleChannel_2 == 5)
                    {
                        VoltageScaleC = VScaleChannel_2 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_2 > 5)
                    {
                        VoltageScaleC = VScaleChannel_2 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_2 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_2;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 3:
                    VScaleChannel_3 = trkVoltageScale.Value;
                    if (VScaleChannel_3 == 5)
                    {
                        VoltageScaleC = VScaleChannel_3 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_3 > 5)
                    {
                        VoltageScaleC = VScaleChannel_3 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_3 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_3;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 4:
                    VScaleChannel_4 = trkVoltageScale.Value;
                    if (VScaleChannel_4 == 5)
                    {
                        VoltageScaleC = VScaleChannel_4 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_4 > 5)
                    {
                        VoltageScaleC = VScaleChannel_4 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_4 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_4;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 5:
                    VScaleChannel_5 = trkVoltageScale.Value;
                    if (VScaleChannel_5 == 5)
                    {
                        VoltageScaleC = VScaleChannel_5 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_5 > 5)
                    {
                        VoltageScaleC = VScaleChannel_5 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_5 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_5;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 6:
                    VScaleChannel_6 = trkVoltageScale.Value;
                    if (VScaleChannel_6 == 5)
                    {
                        VoltageScaleC = VScaleChannel_6 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_6 > 5)
                    {
                        VoltageScaleC = VScaleChannel_6 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_6 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_6;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 7:
                    VScaleChannel_7 = trkVoltageScale.Value;
                    if (VScaleChannel_7 == 5)
                    {
                        VoltageScaleC = VScaleChannel_7 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_7 > 5)
                    {
                        VoltageScaleC = VScaleChannel_7 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_7 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_7;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                case 8:
                    VScaleChannel_8 = trkVoltageScale.Value;
                    if (VScaleChannel_8 == 5)
                    {
                        VoltageScaleC = VScaleChannel_8 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1";
                    }
                    else if (VScaleChannel_8 > 5)
                    {
                        VoltageScaleC = VScaleChannel_8 - 4;
                        lblVoltScale.Text = "Vertical Scale:\n 1/" + VoltageScaleC;
                    }
                    else
                    {
                        //VoltageScaleC = (double)(VScaleChannel_8 / 5.0);
                        //lblVoltScale.Text = VoltageScaleC.ToString() + " Volt\n Per Division";
                        VoltageScaleC = 6 - VScaleChannel_8;
                        lblVoltScale.Text = "Vertical Scale:\n * " + VoltageScaleC;
                    }
                    break;
                default:
                    break;
            }

            Invalidate();

        }

        private void trkVoltageScaleControl_Scroll(object sender, EventArgs e)
        {
            //Set the Select Channel Indicator to Bold and
            //Set the Y position for the Seleted Channel Indicator.
            switch (ChannelSelectControl)
            {
                case 1:
                       VerticalScale_1 = trkVerticalScaleControl.Value;
                       Channel_1_VerticalLabel.Top = (250 - VerticalScale_1);
                       break;
                case 2:
                       VerticalScale_2 = trkVerticalScaleControl.Value;
                       Channel_2_VerticalLabel.Top = (250 - VerticalScale_2);
                       break;
                case 3:
                       VerticalScale_3 = trkVerticalScaleControl.Value;
                       Channel_3_VerticalLabel.Top = (250 - VerticalScale_3);
                       break;
                case 4:
                       VerticalScale_4 = trkVerticalScaleControl.Value;
                       Channel_4_VerticalLabel.Top = (250 - VerticalScale_4);
                       break;
                case 5:
                       VerticalScale_5 = trkVerticalScaleControl.Value;
                       Channel_5_VerticalLabel.Top = (250 - VerticalScale_5);
                       break;
                case 6:
                       VerticalScale_6 = trkVerticalScaleControl.Value;
                       Channel_6_VerticalLabel.Top = (250 - VerticalScale_6);
                       break;
                case 7:
                       VerticalScale_7 = trkVerticalScaleControl.Value;
                       Channel_7_VerticalLabel.Top = (250 - VerticalScale_7);
                       break;
                case 8:
                       VerticalScale_8 = trkVerticalScaleControl.Value;
                       Channel_8_VerticalLabel.Top = (250 - VerticalScale_8);
                       break;

                default:
                    break;
            }

            Invalidate();
        }

        private void trkHorizScaleControl_Scroll(object sender, EventArgs e)
        {
            switch (ChannelSelectControl)
            {
                case 1:
                    HorizontalScale_1 = trkHorizScaleControl.Value;
                    break;
                case 2:
                    HorizontalScale_2 = trkHorizScaleControl.Value;
                    break;
                case 3:
                    HorizontalScale_3 = trkHorizScaleControl.Value;
                    break;
                case 4:
                    HorizontalScale_4 = trkHorizScaleControl.Value;
                    break;
                case 5:
                    HorizontalScale_5 = trkHorizScaleControl.Value;
                    break;
                case 6:
                    HorizontalScale_6 = trkHorizScaleControl.Value;
                    break;
                case 7:
                    HorizontalScale_7 = trkHorizScaleControl.Value;
                    break;
                case 8:
                    HorizontalScale_8 = trkHorizScaleControl.Value;
                    break;

                default:
                    break;
            }
            Invalidate();
        }

        private void trkTimeScale_Scroll(object sender, EventArgs e)
        {
            switch (ChannelSelectControl)
            {
                case 1:
                    TimeScale_1 = trkTimeScale.Value;
                    break;
                case 2:
                    TimeScale_2 = trkTimeScale.Value;
                    break;
                case 3:
                    TimeScale_3 = trkTimeScale.Value;
                    break;
                case 4:
                    TimeScale_4 = trkTimeScale.Value;
                    break;
                case 5:
                    TimeScale_5 = trkTimeScale.Value;
                    break;
                case 6:
                    TimeScale_6 = trkTimeScale.Value;
                    break;
                case 7:
                    TimeScale_7 = trkTimeScale.Value;
                    break;
                case 8:
                    TimeScale_8 = trkTimeScale.Value;
                    break;

                default:
                    break;
            }
            Invalidate();
        }

        private void btnResetTimeScale_Click(object sender, EventArgs e)
        {
            trkTimeScale.Value = 1;

            switch (ChannelSelectControl)
            {
                case 1:
                    TimeScale_1 = trkTimeScale.Value;
                    break;
                case 2:
                    TimeScale_2 = trkTimeScale.Value;
                    break;
                case 3:
                    TimeScale_3 = trkTimeScale.Value;
                    break;
                case 4:
                    TimeScale_4 = trkTimeScale.Value;
                    break;
                case 5:
                    TimeScale_5 = trkTimeScale.Value;
                    break;
                case 6:
                    TimeScale_6 = trkTimeScale.Value;
                    break;
                case 7:
                    TimeScale_7 = trkTimeScale.Value;
                    break;
                case 8:
                    TimeScale_8 = trkTimeScale.Value;
                    break;

                default:
                    break;
            }
            Invalidate();
        }

        public void SetChannelFontToRegular()
        {
            //Set all Channel Indicators to Regular
            if (ActiveChannels[0])
            {
                Channel_1_VerticalLabel.Font = new Font(Channel_1_VerticalLabel.Font, FontStyle.Regular);
                Channel_1_VerticalLabel.Font = new Font("Arial", 8);
            }
            if(ActiveChannels[1])
            {
               Channel_2_VerticalLabel.Font = new Font(Channel_2_VerticalLabel.Font, FontStyle.Regular);
               Channel_2_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[2])
            {
              Channel_3_VerticalLabel.Font = new Font(Channel_3_VerticalLabel.Font, FontStyle.Regular);
              Channel_3_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[3])
            {
               Channel_4_VerticalLabel.Font = new Font(Channel_4_VerticalLabel.Font, FontStyle.Regular);
               Channel_4_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[4])
            {
               Channel_5_VerticalLabel.Font = new Font(Channel_5_VerticalLabel.Font, FontStyle.Regular);
               Channel_5_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[5])
            {
               Channel_6_VerticalLabel.Font = new Font(Channel_6_VerticalLabel.Font, FontStyle.Regular);
               Channel_6_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[6])
            {
               Channel_7_VerticalLabel.Font = new Font(Channel_7_VerticalLabel.Font, FontStyle.Regular);
               Channel_7_VerticalLabel.Font = new Font("Arial", 8);
            }

            if(ActiveChannels[7])
            {
               Channel_8_VerticalLabel.Font = new Font(Channel_8_VerticalLabel.Font, FontStyle.Regular);
               Channel_8_VerticalLabel.Font = new Font("Arial", 8);
            }

        }

        private void btnHorizFastDown_Click(object sender, EventArgs e)
        {
            if (trkHorizScaleControl.Value == 0)
                HorizontalFastShift -= 1;
            else
                HorizontalFastShift = HorizontalFastShift - Math.Abs(trkHorizScaleControl.Value);


            switch (ChannelSelectControl)
            {
                case 1:
                    HorizontalFastShift_1 = HorizontalFastShift;
                    break;
                case 2:
                    HorizontalFastShift_2 = HorizontalFastShift;
                    break;
                case 3:
                    HorizontalFastShift_3 = HorizontalFastShift;
                    break;
                case 4:
                    HorizontalFastShift_4 = HorizontalFastShift;
                    break;
                case 5:
                    HorizontalFastShift_5 = HorizontalFastShift;
                    break;
                case 6:
                    HorizontalFastShift_6 = HorizontalFastShift;
                    break;
                case 7:
                    HorizontalFastShift_7 = HorizontalFastShift;
                    break;
                case 8:
                    HorizontalFastShift_8 = HorizontalFastShift;
                    break;

                default:
                    break;
            }
            Invalidate();
        }

        private void btnHorizFastUp_Click(object sender, EventArgs e)
        {
                HorizontalFastKeyUp();
        }

        void HorizontalFastKeyUp()
        {
            if (trkHorizScaleControl.Value == 0)
                HorizontalFastShift += 1;
            else
                HorizontalFastShift = HorizontalFastShift + Math.Abs(trkHorizScaleControl.Value);


            switch (ChannelSelectControl)
            {
                case 1:
                    HorizontalFastShift_1 = HorizontalFastShift;
                    break;
                case 2:
                    HorizontalFastShift_2 = HorizontalFastShift;
                    break;
                case 3:
                    HorizontalFastShift_3 = HorizontalFastShift;
                    break;
                case 4:
                    HorizontalFastShift_4 = HorizontalFastShift;
                    break;
                case 5:
                    HorizontalFastShift_5 = HorizontalFastShift;
                    break;
                case 6:
                    HorizontalFastShift_6 = HorizontalFastShift;
                    break;
                case 7:
                    HorizontalFastShift_7 = HorizontalFastShift;
                    break;
                case 8:
                    HorizontalFastShift_8 = HorizontalFastShift;
                    break;

                default:
                    break;
            }
            Invalidate();

        }

        private void btnResetHorizPos_Click(object sender, EventArgs e)
        {
            trkHorizScaleControl.Value = 0;
            HorizontalFastShift = 0;

            switch (ChannelSelectControl)
            {
                case 1:
                    HorizontalFastShift_1 = 0;
                    HorizontalScale_1 = 0;
                    break;
                case 2:
                    HorizontalFastShift_2 = 0;
                    HorizontalScale_2 = 0;
                    break;
                case 3:
                    HorizontalFastShift_3 = 0;
                    HorizontalScale_3 = 0;
                    break;
                case 4:
                    HorizontalFastShift_4 = 0;
                    HorizontalScale_4 = 0;
                    break;
                case 5:
                    HorizontalFastShift_5 = 0;
                    HorizontalScale_5 = 0;
                    break;
                case 6:
                    HorizontalFastShift_6 = 0;
                    HorizontalScale_6 = 0;
                    break;
                case 7:
                    HorizontalFastShift_7 = 0;
                    HorizontalScale_7 = 0;
                    break;
                case 8:
                    HorizontalFastShift_8 = 0;
                    HorizontalScale_8 = 0;
                    break;

                default:
                    break;
            }
            Invalidate();
        }

        private void btnChannelOnOff_Click(object sender, EventArgs e)
        {
            switch (ChannelSelectControl)
            {
                case 1:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_1_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_1_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 2:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_2_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_2_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 3:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_3_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_3_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 4:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_4_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_4_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 5:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_5_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_5_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 6:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_6_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_6_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 7:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_7_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_7_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;
                case 8:
                    if (ActiveChannels[ChannelSelectControl-1])
                    {
                        ActiveChannels[ChannelSelectControl-1] = false;
                        btnChannelOnOff.Text = "Off";
                        Channel_8_VerticalLabel.Font = new Font("Arial", 4);
                    }
                    else if (ScopeBuffer[ChannelSelectControl-1] != null)
                    {
                        ActiveChannels[ChannelSelectControl-1] = true;
                        btnChannelOnOff.Text = "On";
                        Channel_8_VerticalLabel.Font = new Font("Arial", 8);
                    }
                    break;

                default:
                    break;
            }
            Invalidate();

        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }



    }
}
