﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

namespace HyperSerialPort
{
    public partial class frmExtTriggerForm : Form
    {
        public static string ExtTriggerPortName = "COM1";
        public static string ExtTriggerSourceBaudRate = "9600";
        public static string ExtTriggerSourceDataBits = "8";
        public static string ExtTriggerSourceStopBits = "1";
        public static string ExtTriggerSourceHandshake = "XOnXOff";
        public static string ExtTriggerSourceParity = "None";

        //Internal registers
        public static bool ExtTriggerInitialized = false;

        
        public frmExtTriggerForm()
        {
            InitializeComponent();

            // Get a list of serial port names.
            string[] ports;
            ports = SerialPort.GetPortNames();

            // Display each port name to the console.
            foreach (string port in ports)
            {
                comboBoxSource.Items.Add(port);
            }
         }

        //creating object of parent to access shared variable  
        HyperSerialPort frm = new HyperSerialPort();

        //Char array to grab the characters from the ExtTriggerCharacterComboBox
        public char[] ExtTriggerCharacter;

        public string FormExtTriggerSourcePortName
        {
            get
            {
                return (ExtTriggerPortName);
            }
            set
            {
                ExtTriggerPortName = value;
            }
        }

        public string FormExtTriggerSourceBaudRate
        {
            get
            {
                return (ExtTriggerSourceBaudRate);
            }
            set
            {
                ExtTriggerSourceBaudRate = value;
            }
        }

        public string FormExtTriggerSourceDataBits
        {
            get
            {
                return (ExtTriggerSourceDataBits);
            }
            set
            {
                ExtTriggerSourceDataBits = value;
            }
        }

        public string FormExtTriggerSourceParity
        {
            get
            {
                return (ExtTriggerSourceParity);
            }
            set
            {
                ExtTriggerSourceParity = value;
            }
        }

        public string FormExtTriggerSourceStopBits
        {
            get
            {
                return (ExtTriggerSourceStopBits);
            }
            set
            {
                ExtTriggerSourceStopBits = value;
            }
        }

        public string FormExtTriggerSourceHandshake
        {
            get
            {
                return (ExtTriggerSourceHandshake);
            }
            set
            {
                ExtTriggerSourceHandshake = value;
            }
        }
        
        private void btnOk_Click(object sender, EventArgs e)
        {
            if (serialPort2.IsOpen)
                serialPort2.Close();

            frm.ExtTriggerForm_ExtTriggerWaiting = false;
            ExtTriggerInitialized = true;
            this.Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (serialPort2.IsOpen)
                serialPort2.Close();

            frm.ExtTriggerForm_ExtTriggerWaiting = false;
            ExtTriggerInitialized = true;
            this.Close();
        }

        private void comboBoxSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            ExtTriggerPortName = comboBoxSource.Text;
            ShowExtTriggerSourceForm();
        }

        public void ShowExtTriggerSourceForm()
        {
            //This instance allows communications with the child listbox.
            //frmChildForm newChildForm = new frmChildForm(this.listBox1);
            frmExtTriggerSourceForm newChildForm = new frmExtTriggerSourceForm();
            newChildForm.ShowDialog();
            newChildForm.Dispose();

        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            //Turn on the Serial Port 2
            if (!ExtTriggerInitialized)
            {
                OpenSerialPort2();
                ExtTriggerInitialized = true;
            }
            frm.ExtTriggerForm_ExtTriggerWaiting = true;
        }
        public void OpenSerialPort2()
        {
            try
            {
                serialPort2.PortName = ExtTriggerPortName;
                serialPort2.BaudRate = Convert.ToInt32(ExtTriggerSourceBaudRate);
                serialPort2.Parity = (Parity)Enum.Parse(typeof(Parity), ExtTriggerSourceParity);
                serialPort2.DataBits = Convert.ToInt16(ExtTriggerSourceDataBits);
                serialPort2.StopBits = (StopBits)Enum.Parse(typeof(StopBits), ExtTriggerSourceStopBits);
                serialPort2.Handshake = (Handshake)Enum.Parse(typeof(Handshake), ExtTriggerSourceHandshake); ;
                //Check if serialPort2 is closed
                if (!serialPort2.IsOpen)
                {
                    //Open serialPort2 and set the ExtTriggerWaiting flag to true.
                    //This will prevent serialPort1 from printing characters to the screen.
                    serialPort2.Open();
                    frm.ExtTriggerForm_ExtTriggerWaiting = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void serialPort2_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //When a character is recieved, launch the external trigger process 
            //and check for the trigger character
            this.Invoke(new EventHandler(ExtTriggerProcess));
        }

        private void ExtTriggerProcess(object sender, EventArgs e)
        {
            if (serialPort2.ReadByte() == (byte)ExtTriggerCharacter[0])
            {
                frm.ExtTriggerForm_ExtTriggerWaiting = false;
                //this.Invoke(new EventHandler(frm.ExtTriggerDetected));
            }
        }

        private void textBoxCharacter_TextChanged(object sender, EventArgs e)
        {
            ExtTriggerCharacter = textBoxCharacter.Text.ToCharArray(0,textBoxCharacter.Text.Length);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
                if (serialPort2.IsOpen)
                    serialPort2.Close();

                frm.ExtTriggerForm_ExtTriggerWaiting = false;
                ExtTriggerInitialized = true;
        }

    }
}
