﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Microsoft.Win32;



namespace HyperSerialPort
{
    public partial class HyperSerialPort : Form
    {

        //Set #define's here
        public const int EXTTRIGGERTIMEOUT = 100000;
        public const double HSP_VERSION_NUMBER = 0.220;
        public const int SEND_CHAR_OPEN_PORT = 0;
        public const int SEND_CHAR_ONE_SHOT = 1;
        public const int SEND_CHAR_RUN_TEST = 2;
        public const int SEND_CHAR_DISPLAY_REPITITIVE = 0; 
        public const int SEND_CHAR_DISPLAY_ONE_SHOT = 1;
        public const int SEND_CHAR_DISPLAY_IDLE = 2;
        public const int STOP_WATCH_TOOLSTRIP_LABEL  = 0;
        public const int STOP_WATCH_TEXTBOX1 = 1;
        
        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void DisplayCharactersCallback(string text);
        delegate void SendCharactersCallback(string text);

        // This thread is used to demonstrate both thread-safe and
        // unsafe ways to call a Windows Forms control.
        private Thread DisplayThread = null;
        private Thread SendThread = null;

        // Serial port 1 received string
        string CallGetDataChar;

        // Serial port send string
        char[] PutBuff;

        // Local Serial Port parameters From Properties Page
        public static string PortName="COM1";
        public static string BaudRate="9600";
        public static string DataBits = "8";
        public static string vParity = "None";
        public static string StopBits = "1";
        public static string pHandshake = "XOnXOff";

        //ASCII Modify serial stream From Properties Page
        public static bool AddLineFeedCharacter = false;
        public static bool EchoTransmitCharacters = true;
        public static bool AppendLineFeed = false;
        public static bool Force7bit = false;
        public static bool WrapLines = false;
        public static int LineDelay = 0;
        public static int CharacterDelay = 0;

        // Flag to determine if serial port should be closed from Properties Page
        public static bool propertiesCloseSerialPort = false;

        // Flags to open and close the serial port
        public static bool OpenSFPort = false;

        //Flag  for converting incoming stream to Hex
        public static bool HexConversion = false;
        public static int IdxHexChar = 0;
        public static bool hexConversionTabFlag = false;

        //Stopwatch class and user variable for 
        //Global Timer
        Stopwatch stopWatch = new Stopwatch();

        //Millisecond Timer for the Send Char Test
        System.Threading.Timer sendchar_Timer;

        //Microsecond Timer for the Send Char Test
        MicroLibrary.MicroTimer microTimer = new MicroLibrary.MicroTimer();


        //Variables for Displaying time of connection
        public static int LastTimerTickSeconds = 0;

        //Variables to pass the Send Character Test Tools parameters
        public static string SendCharString = "";
        public static int SendTimerInterval = 0;
        public static bool SendCharTest = false;
        public static bool SendCharMicroTimerStartFlag = false;

        //Variables to make the Send Timer Characters work
        public static int sendCharState = 0;
        public static int sendCharDisplayState = 0;
        public static int LastSendTimerCapture = 0;
        public static bool SendCharTestStartFlag = true;
        public static bool millisecondTimerFlag = false;
        public static bool SendCharOneShot = false;
        public static bool SendCharRepetitive = false;
        public static bool SendCharIncrement7Bit = false;
        public static bool SendCharIncrementByte = false;
        public static bool sendCharOneShotFire = false;

        //Temp for Test
        //public static bool SendCharTestFlag = true;

        //External Trigger Flags
        public static bool ExtTriggerWaiting = false;

        //Flag to hold the serial port status before power suspend
        public static bool serialPort1RunningBeforeSuspend = false;

        //Version Number registers
        public static double hspVersionNumber = HSP_VERSION_NUMBER;

        // This thread is used to demonstrate both thread-safe and
        // unsafe ways to call a Windows Forms control.
        private Thread DisplayThreadMultiWindow = null;
        private Thread SendThreadMultiWindow = null;

        // This delegate enables asynchronous calls for setting
        // the text property on a MultiWindowTextBox control.
        delegate void DisplayCharactersCallbackMultiWindow(string text);

        //Multi Terminal Textbox declaration
        TextBox multiTerminalTextbox = null;
        Button multiTerminalConnect = null;
        Button multiTerminalDisconnect = null;
        Button multiTerminalConfigure = null;
        Button multiTerminalClose = null;
        Label multiTerminalLabel = null;
        ToolStripStatusLabel toolStripStatusLabel4 = null;
        ToolStripStatusLabel toolStripStatusLabel5 = null;

        // Multi Terminal Serial Port parameters
        public static string mtPortName = "COM1";
        public static string mtBaudRate = "9600";
        public static string mtDataBits = "8";
        public static string mtParity = "None";
        public static string mtStopBits = "1";
        public static string mtHandshake = "XOnXOff";
        public static bool mtStartMultiWindow = false;

        //Multi Terminal receive string
        public string CallGetDataCharMultiWindow = "";
        public char [] PutBuffMultiWindow;

        public HyperSerialPort()
        {
            InitializeComponent();

            //Set text window and buttons to initial conditions
            buttonStart.Enabled = true;
            buttonStop.Enabled = false;
            textBox1.ReadOnly = true;

            //Set the toolStripStatusLabel2.Text to some initial values
            toolStripStatusLabel1.Text = "DisConnected ";
            toolStripStatusLabel2.Text = PortName + "  " + BaudRate + " - " + DataBits + " - " + StopBits;
            toolStripStatusLabel3.Text = "ASCII";

            //Register the PowerModeChanged Event, to detect when system goes into suspend or hibernation
            SystemEvents.PowerModeChanged += new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);

        }

        public string Form2_PortName  
        {
            get
            {
                return (PortName);
            }
            set 
             {
                 PortName = value;   
             }
        }
        public string Form2_BaudRate
        {
            get
            {
                return (BaudRate);
            }
            set
            {
                BaudRate = value;
            }
        }

        public string Form2_DataBits
        {
            get
            {
                return (DataBits);
            }
            set
            {
                DataBits = value;
            }
        }

        public string Form2_Parity
        {
            get
            {
                return (vParity);
            }
            set
            {
                vParity = value;
            }
        }

        public string Form2_StopBits
        {
            get
            {
                return (StopBits);
            }
            set
            {
                StopBits = value;
            }
        }

        public string Form2_Handshake
        {
            get
            {
                return (pHandshake);
            }
            set
            {
                pHandshake = value;
            }
        }

        public bool Form2_EchoTransmitCharacters
        {
            get
            {
                return (EchoTransmitCharacters);
            }
            set
            {
                EchoTransmitCharacters = value;
            }
        }

        public bool Form2_AddLineFeedCharacter
        {
            get
            {
                return (AddLineFeedCharacter);
            }
            set
            {
                AddLineFeedCharacter = value;
            }
        }

        public int Form2_LineDelay
        {
            get
            {
                return (LineDelay);
            }
            set
            {
                LineDelay = value;
            }
        }

        public int Form2_CharacterDelay
        {
            get
            {
                return (CharacterDelay);
            }
            set
            {
                CharacterDelay = value;
            }
        }

        public bool Form2_AppendLineFeed
        {
            get
            {
                return (AppendLineFeed);
            }
            set
            {
                AppendLineFeed = value;
            }
        }

        public bool Form2_Force7bit
        {
            get
            {
                return (Force7bit);
            }
            set
            {
                Force7bit = value;
            }
        }

        public bool Form2_WrapLines
        {
            get
            {
                return (WrapLines);
            }
            set
            {
                WrapLines = value;
            }
        }


        public bool Form2_propertiesCloseSerialPort
        {
            get
            {
                return (propertiesCloseSerialPort);
            }
            set
            {
                propertiesCloseSerialPort = value;
            }
        }

        public bool SendFile_OpenSFPort
        {
            get
            {
                return (OpenSFPort);
            }
            set
            {
                OpenSFPort = value;
            }
        }

        public string SendCharactersForm_SendCharString
        {
            get
            {
                return (SendCharString);
            }
            set
            {
                SendCharString = value;
            }
        }

        public int SendCharactersForm_SendTimerInterval
        {
            get
            {
                return (SendTimerInterval);
            }
            set
            {
                SendTimerInterval = value;
            }
        } 

        public bool  SendCharactersForm_SendCharTest
        {
            get
            {
                return (SendCharTest);
            }
            set
            {
                SendCharTest = value;
            }
        } 

        public bool  SendCharactersForm_millisecondTimerFlag
        {
            get
            {
                return (millisecondTimerFlag);
            }
            set
            {
                millisecondTimerFlag = value;
            }
        } 

        public bool  SendCharactersForm_SendCharMicroTimerStartFlag
        {
            get
            {
                return (SendCharMicroTimerStartFlag);
            }
            set
            {
                SendCharMicroTimerStartFlag = value;
            }
        }

        public bool SendCharactersForm_SendCharOneShot
        {
            get
            {
                return (SendCharOneShot);
            }
            set
            {
                SendCharOneShot = value;
            }
        }
        public bool SendCharactersForm_SendCharRepetitive
        {
            get
            {
                return (SendCharRepetitive);
            }
            set
            {
                SendCharRepetitive = value;
            }
        }
        public bool SendCharactersForm_SendCharIncrement7Bit
        {
            get
            {
                return (SendCharIncrement7Bit);
            }
            set
            {
                SendCharIncrement7Bit = value;
            }
        }
        
        public bool SendCharactersForm_SendCharIncrementByte
        {
            get
            {
                return (SendCharIncrementByte);
            }
            set
            {
                SendCharIncrementByte = value;
            }
        }

        public bool ExtTriggerForm_ExtTriggerWaiting
        {
            get
            {
                return (ExtTriggerWaiting);
            }
            set
            {
                ExtTriggerWaiting = value;
            }
        } 

        public double AboutHSP_hspVersionNumber
        {
            get
            {
                return (hspVersionNumber);
            }
            set
            {
                hspVersionNumber = value;
            }
        }

        public string MultiTerminal_PortName
        {
            get
            {
                return (mtPortName);
            }
            set
            {
                mtPortName = value;
            }
        }
        public string MultiTerminal_BaudRate
        {
            get
            {
                return (mtBaudRate);
            }
            set
            {
                mtBaudRate = value;
            }
        }

        public string MultiTerminal_DataBits
        {
            get
            {
                return (mtDataBits);
            }
            set
            {
                mtDataBits = value;
            }
        }

        public string MultiTerminal_Parity
        {
            get
            {
                return (mtParity);
            }
            set
            {
                mtParity = value;
            }
        }

        public string MultiTerminal_StopBits
        {
            get
            {
                return (mtStopBits);
            }
            set
            {
                mtStopBits = value;
            }
        }

        public string MultiTerminal_Handshake
        {
            get
            {
                return (mtHandshake);
            }
            set
            {
                mtHandshake = value;
            }
        }

        public bool MultiTerminal_StartMultiWindow
        {
            get
            {
                return (mtStartMultiWindow);
            }
            set
            {
                mtStartMultiWindow = value;
            }
        }


        [Serializable()]	//Set this attribute to all the classes that you define to be serialized
        public class HSPortLoadTypes : ISerializable
        {
            public string HSPortLoadTypes_PortName;
            public string HSPortLoadTypes_BaudRate;
            public string HSPortLoadTypes_DataBits;
            public string HSPortLoadTypes_vParity;
            public string HSPortLoadTypes_StopBits;
            public string HSPortLoadTypes_pHandshake;

            //Default constructor
            public HSPortLoadTypes()
            {
                HSPortLoadTypes_PortName = null;
                HSPortLoadTypes_BaudRate = null;
                HSPortLoadTypes_DataBits = null;
                HSPortLoadTypes_vParity = null;
                HSPortLoadTypes_StopBits = null;
                HSPortLoadTypes_pHandshake = null;
            }

            //Deserialization constructor.
            public HSPortLoadTypes(SerializationInfo info, StreamingContext ctxt)
            {
                //Get the values from info and assign them to the appropriate properties
                HSPortLoadTypes_PortName = (String)info.GetValue("HSPortLoadTypes_PortName", typeof(string));
                HSPortLoadTypes_BaudRate = (String)info.GetValue("HSPortLoadTypes_BaudRate", typeof(string));
                HSPortLoadTypes_DataBits = (String)info.GetValue("HSPortLoadTypes_DataBits", typeof(string));
                HSPortLoadTypes_vParity = (String)info.GetValue("HSPortLoadTypes_vParity", typeof(string));
                HSPortLoadTypes_StopBits = (String)info.GetValue("HSPortLoadTypes_StopBits", typeof(string));
                HSPortLoadTypes_pHandshake = (String)info.GetValue("HSPortLoadTypes_pHandshake", typeof(string));
            }

            //Serialization function.
            public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
            {
                //You can use any custom name for your name-value pair. But make sure you
                // read the values with the same name. For ex:- If you write EmpId as "EmployeeId"
                // then you should read the same with "EmployeeId"
                info.AddValue("HSPortLoadTypes_PortName", HSPortLoadTypes_PortName);
                info.AddValue("HSPortLoadTypes_BaudRate", HSPortLoadTypes_BaudRate);
                info.AddValue("HSPortLoadTypes_DataBits", HSPortLoadTypes_DataBits);
                info.AddValue("HSPortLoadTypes_vParity", HSPortLoadTypes_vParity);
                info.AddValue("HSPortLoadTypes_StopBits", HSPortLoadTypes_StopBits);
                info.AddValue("HSPortLoadTypes_pHandshake", HSPortLoadTypes_pHandshake);
            }
        }

        private void mnuCut_Click(object sender, EventArgs e)
        {
             if (textBox1.SelectedText != "")
             {
                 textBox1.Cut();
             }
       }

       private void mnunUdo_Click(object sender, EventArgs e)
       {
           if (textBox1.CanUndo == true)
           {
               textBox1.Undo();
               textBox1.ClearUndo();
           }
       }

       private void mnuCopy_Click(object sender, EventArgs e)
       {
           if (textBox1.SelectionLength > 0)
           {
               textBox1.Copy();
           }
       }

       private void mnuPaste_Click(object sender, EventArgs e)
       {
           if (Clipboard.GetDataObject().GetDataPresent(DataFormats.Text) == true)
           {
               textBox1.Paste();
           }
       }

       private void richTextBox1_TextChanged(object sender, EventArgs e)
       {
       }


        private void buttonOpenFile_Click(object sender, EventArgs e)
        {
            mnuOpen_Click(sender, e);
        }

        private void mnuFont_Click(object sender, EventArgs e)
        {
            fontDialog1.ShowColor = true;
            fontDialog1.ShowApply = true;
            fontDialog1.ShowEffects = true;
            fontDialog1.ShowHelp = true;

            fontDialog1.MaxSize = 32;
            fontDialog1.MinSize = 6;

            if (fontDialog1.ShowDialog() != DialogResult.Cancel)
            {
                textBox1.Font = fontDialog1.Font;
                textBox1.BackColor = fontDialog1.Color;
            }
        }
        
        private void mnuOpen_Click(object sender, EventArgs e)
        {
            string Chosen_file = "";
			//Create a new HSPortLoadTypes object
            HSPortLoadTypes mp = new HSPortLoadTypes();

            openFD.InitialDirectory = "C:";
            openFD.Title = "Select the HyperSerialPort Initialization File";
            openFD.FileName = "";
            openFD.Filter = "HyperSerialPort File|*.hsp";

            if (openFD.ShowDialog() == DialogResult.Cancel)
            {
                MessageBox.Show("File Open Cancelled");
                return;
            }
            else
            {
                Chosen_file = openFD.FileName;
            }

            //Open the file written above and read values from it.
            Stream stream = File.Open(Chosen_file, FileMode.Open);
            BinaryFormatter bformatter = new BinaryFormatter();


            mp = (HSPortLoadTypes)bformatter.Deserialize(stream);
			stream.Close();

            PortName = mp.HSPortLoadTypes_PortName.ToString();
            BaudRate = mp.HSPortLoadTypes_BaudRate;
            DataBits =  mp.HSPortLoadTypes_DataBits.ToString();
            vParity = mp.HSPortLoadTypes_vParity;
            StopBits = mp.HSPortLoadTypes_StopBits.ToString();
            pHandshake = mp.HSPortLoadTypes_pHandshake;

        }

        private void buttonSaveFile_Click(object sender, EventArgs e)
        {
            mnuSave_Click(sender, e);
        }

        private void mnuSave_Click(object sender, EventArgs e)
        {
            string Saved_File = "";
            HSPortLoadTypes mp = new HSPortLoadTypes();

            mp.HSPortLoadTypes_PortName = PortName;
            mp.HSPortLoadTypes_BaudRate = BaudRate;
            mp.HSPortLoadTypes_DataBits = DataBits;
            mp.HSPortLoadTypes_vParity = vParity;
            mp.HSPortLoadTypes_StopBits = StopBits;
            mp.HSPortLoadTypes_pHandshake = pHandshake;

            saveFD.InitialDirectory = "C:";
            saveFD.Title = "Save a HyperSerialPort File";
            saveFD.FileName = "";

            saveFD.Filter = "HyperSerialPort Files|*.hsp";

            if (saveFD.ShowDialog() != DialogResult.Cancel)
            {
                Saved_File = saveFD.FileName;
            }
            else
                return;

            // Open a file and serialize the object into it in binary format.
            // HyperSerialPort.hsp is the file that we are creating. 
            // If you use custom extensions, then the user will know 
            //   that the file is associated with your program.
            Stream stream = File.Open(Saved_File, FileMode.Create);

            BinaryFormatter bformatter = new BinaryFormatter();

            bformatter.Serialize(stream, mp);
            stream.Close();
        }




       private void mnuQuit_Click_1(object sender, EventArgs e)
       {
           if (MessageBox.Show("Quit HyperSerialPort?", "Exit", MessageBoxButtons.OKCancel) == DialogResult.OK)
           {
               Application.Exit();
           }

       }


       private void buttonStart_Click(object sender, EventArgs e)
       {
           OpenSerialPort1();
           //Turn on the stopwatch to allow display of elapsed time
           if(!timer1.Enabled)
                timer1.Enabled = true;
           stopWatch.Start();

       }
       public void OpenSerialPort1()
       {
           try
           {
               serialPort1.PortName = PortName;
               serialPort1.BaudRate = Convert.ToInt32(BaudRate);
               serialPort1.Parity = (Parity)Enum.Parse(typeof(Parity), vParity);
               serialPort1.DataBits = Convert.ToInt16(DataBits);
               serialPort1.StopBits = (StopBits)Enum.Parse(typeof(StopBits), StopBits);
               serialPort1.Handshake = (Handshake)Enum.Parse(typeof(Handshake), pHandshake);
               //serialPort1.Handshake = SetPortHandshake;
               if (!serialPort1.IsOpen)
               {
                   serialPort1.Open();
                   buttonStart.Enabled = false;
                   buttonStop.Enabled = true;
                   textBox1.ReadOnly = false;
                   toolStripStatusLabel2.Text = PortName + "  " + BaudRate + " - " + DataBits + " - " + StopBits;
               }
           }
           catch (Exception ex)
           {
               MessageBox.Show(ex.Message);
           }
       }

       private void buttonStop_Click(object sender, EventArgs e)
       {
           CloseSerialPort1();
       }

       public void CloseSerialPort1()
       {
               //Close the serial port and window
               serialPort1.Close();
               buttonStart.Enabled = true;
               buttonStop.Enabled = false;
               textBox1.ReadOnly = true;

               //Stop the Stopwatch, disable the timer and set time to 0
               stopWatch.Stop();
               stopWatch.Reset();
               LastTimerTickSeconds = 0;
               //timer1.Enabled = false;
               toolStripStatusLabel1.Text = "Disconnected ";
       }

       //When the system goes into suspend or hibernation, stop the stopwatch
       private void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e)
       {
           if (e.Mode == PowerModes.Suspend)
           {
               //Close the serial port, otherwise Windows will lock up.
               //Also, set a flag to notify HSP to turn serial port on
               //when PC comes out of suspend.
               if (serialPort1.IsOpen)
               {
                   serialPort1.Close();
                   stopWatch.Stop();
                   buttonStart.Enabled = true;
                   buttonStop.Enabled = false;
                   serialPort1RunningBeforeSuspend = true;
                   timer1.Enabled = false;
               }

               toolStripStatusLabel1.Text = "Suspend";
           }

           if (e.Mode == PowerModes.Resume)
           {
               //Check the serialPort1RunningBeforeSuspend flag, if
               //set, then open serial port and resume stopWatch.
               if (serialPort1RunningBeforeSuspend)
               {
                   serialPort1.Open();
                   LastTimerTickSeconds = 0;
                   timer1.Enabled = true;
                   stopWatch.Start();
                   buttonStart.Enabled = false;
                   buttonStop.Enabled = true;
                   serialPort1RunningBeforeSuspend = false;
                   toolStripStatusLabel1.Text = "Connected";
               }
               else
                    toolStripStatusLabel1.Text = "Disconnected";
           }

           return;
       }

       private void Form1_FormClosing(object sender, FormClosingEventArgs e)
       {
               //Close the serial port and stop the timer1
               serialPort1.Close();

               //Stop the Stopwatch, disable the timer and set time to 0
               stopWatch.Stop();
               stopWatch.Reset();
               timer1.Enabled = false;
               toolStripStatusLabel1.Text = "Disconnected ";
       }

       private void textBox1_KeyPress(object sender, KeyPressEventArgs e)
       {

           // If the port is closed, don't try to send a character.
           if (!serialPort1.IsOpen) return;

           // If the port is Open, declare a char[] array with one element.
            PutBuff = new char[1];

           // Load element 0 with the key character.
               PutBuff[0] = e.KeyChar;

           // Send the one character buffer.
           this.SendThread = new Thread(new ThreadStart(this.CallPutData));
           this.SendThread.Start();

           // Set the KeyPress event as handled so the character won't
           // display locally. If you want it to display, omit the next line.
           e.Handled = EchoTransmitCharacters;
       }


      private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
       {
           //dataready = true;
           //backgroundWorker1.RunWorkerAsync();
           this.Invoke(new EventHandler(CallGetData));
       }


        public void ExtTriggerDetected(object sender, EventArgs e)
        {
            int ExtTriggerTimeOutCntr = 0;
            //if (!serialPort1.IsOpen)
            //    OpenSerialPort1();
            //this.serialPort1.DataReceived -= new System.IO.Ports.SerialDataReceivedEventHandler(this.serialPort1_DataReceived);

            while (serialPort1.BytesToRead == 0)
            {
                //Wait until characters are received at the serial port 1
                //Use a timeout counter to prevent infinite loop
                if (ExtTriggerTimeOutCntr++ > EXTTRIGGERTIMEOUT)
                {
                    MessageBox.Show("External Trigger Timeout due to no bytes received.");
                    //if (serialPort1.IsOpen)
                    //     serialPort1.Close();
                    return;
                }
                else
                    textBox1.AppendText("Received incoming characters at:");
                    //DisplayStopWatchElapsedTime(STOP_WATCH_TEXTBOX1);
            }

            //If serial port did not time out, then call the event handler CallGetData()
            //this.Invoke(new EventHandler(CallGetData));


        }

       //CallGetData is invoked to process the data in the serial port 1 buffer
       private void CallGetData(object sender, EventArgs e)
       {
           while (serialPort1.BytesToRead != 0)
           {
               //CallGetDataChar gets all characters in the serial port buffer.
               //Display thread is used to invoke a thread to display while not 
               //interrupting the reading of the serial port.
               CallGetDataChar = serialPort1.ReadExisting();
               if (!ExtTriggerWaiting)
               {
                   this.DisplayThread = new Thread(new ThreadStart(this.ThreadProcDisplay));
                   this.DisplayThread.Start();
               }
           }
       }
       // This method is executed on the worker thread and makes
       // a thread-safe call on the TextBox control.
       private void ThreadProcDisplay()
       {
           //this.DisplayCharacters(Convert.ToChar(CallGetDataChar));
           this.DisplayCharacters(CallGetDataChar);

       }

       //DisplayCharacters() is thread safe method to display characters
        // from serial port on the main textbox
       private void DisplayCharacters(string text)
       {
           // InvokeRequired required compares the thread ID of the
           // calling thread to the thread ID of the creating thread.
           // If these threads are different, it returns true.
           if (this.textBox1.InvokeRequired)
           {
               DisplayCharactersCallback d = new DisplayCharactersCallback(DisplayCharacters);
               this.Invoke(d, new object[] { text });
           }
           else
           {
               if (HexConversion)
               {
                   //Perform a hex conversion on each char in the string and display it.
                   string HexChar;
                   foreach (char c in text)
                   {
                       int tmp = c;
                       HexChar = String.Format("{0:x2}", (uint)System.Convert.ToUInt32(tmp.ToString()));
                       if (c > 0x7f)
                           textBox1.AppendText("0x" + HexChar + " = 'Binary'");
                       else if (c > 0x20)
                           textBox1.AppendText("0x" + HexChar + " = '" + c + "'" );
                       else
                       {
                           ASCII_To_Hex.Conversion asciiDecode;
                           asciiDecode = new ASCII_To_Hex.Conversion();
                           textBox1.AppendText("0x" + HexChar + " = '" + asciiDecode.HexMap(c) + "'" );
                           hexConversionTabFlag = true;
                       }
                       if (IdxHexChar++ < 4)
                       {
                           if (hexConversionTabFlag)
                           {
                               textBox1.AppendText("\t");
                               hexConversionTabFlag = false;
                           }
                           else
                               textBox1.AppendText("\t\t");
                       }
                       else
                       {
                           textBox1.AppendText("\r\n");
                           IdxHexChar   = 0;
                       }
                   }
               }
               else
               {
                   //If HexConversion is false display each char in string.
                 
                   if (AppendLineFeed)
                   {
                       foreach (char c in text)
                       {
                           switch(c)
                           {
                              case '\n':
                               this.textBox1.AppendText("\r\n");
                               break;
                              default:
                               this.textBox1.AppendText(text);
                               break;
                            }
                       }
                   }
                   else
                  
                       this.textBox1.AppendText(text);
               }
           }
       }

       //CallPutData writes characters to the serial port
       //It can be used by thread safe calls.
       private void CallPutData()
       {
           if (!AddLineFeedCharacter)
               serialPort1.Write(PutBuff, 0, 1);
           else
           {
               if (PutBuff[0] == '\r')
               {
                   serialPort1.Write(PutBuff, 0, 1);
                   Thread.Sleep(100);
                   PutBuff[0] = '\n';
                   serialPort1.Write(PutBuff, 0, 1);
               }
           }
       }
        
       private void mnuDisconnect_Click(object sender, EventArgs e)
       {
           if (serialPort1.IsOpen)
               CloseSerialPort1();
       }

       private void buttonProperties_Click(object sender, EventArgs e)
       {
           //if (!serialPort1.IsOpen)
               mnuProperties_Click(sender, e);
           //else
           //    MessageBox.Show("Must Close Serial Port Before Accessing Properties");

       }

       //This is an alias to the Properties menu item under File
       private void mnuProperties_Click(object sender, EventArgs e)
       {
           ShowChildForm();

       }


       public void ShowChildForm()
       {
           //This instance allows communications with the child listbox.
           //frmChildForm newChildForm = new frmChildForm(this.listBox1);
           frmChildForm newChildForm = new frmChildForm();
           newChildForm.ShowDialog();
           newChildForm.Dispose();

           //Call the close serial port 1 if the properties window was called
           if (propertiesCloseSerialPort)
           {
               CloseSerialPort1();
               toolStripStatusLabel2.Text = PortName + "  " + BaudRate + " - " + DataBits + " - " + StopBits;
               propertiesCloseSerialPort = false;
           }
       }

       private void mnuWaitForCall_Click(object sender, EventArgs e)
       {
           serialPort1.Open();
           if (serialPort1.IsOpen)
           {
               buttonStart.Enabled = false;
               buttonStop.Enabled = true;
               textBox1.ReadOnly = false;
           }
       }

       private void mnuStopWaiting_Click(object sender, EventArgs e)
       {
           serialPort1.Close();
           buttonStart.Enabled = true;
           buttonStop.Enabled = false;
           textBox1.ReadOnly = true;
       }

       private void buttonSendText_Click(object sender, EventArgs e)
       {
           mnuSendFile_Click(sender, e);
       }
       private void mnuSendTextFile_Click(object sender, EventArgs e)
       {
           mnuSendFile_Click(sender, e);
       }
       private void mnuSendFile_Click(object sender, EventArgs e)
       {
           SendFileForm();
       }

       public void SendFileForm()
       {
           serialPort1.Close();
           buttonStart.Enabled = true;
           buttonStop.Enabled = false;
           //This instance allows communications with the child listbox.
           //frmChildForm newChildForm = new frmChildForm(this.listBox1);
           frmSendFileForm newSendFileForm = new frmSendFileForm();
           newSendFileForm.ShowDialog(); 
           newSendFileForm.Dispose();

       }

       private void buttonCapture1Text_Click(object sender, EventArgs e)
       {
           mnuReceiveFile_Click(sender, e);
       }
       private void mnuCaptureText_Click(object sender, EventArgs e)
       {
           mnuReceiveFile_Click(sender, e);
       }
       private void mnuReceiveFile_Click(object sender, EventArgs e)
       {
           ReceiveFileForm();
       }

       public void ReceiveFileForm()
       {
           serialPort1.Close();
           buttonStart.Enabled = true;
           buttonStop.Enabled = false;
           //This instance allows communications with the child listbox.
           frmReceiveFileForm newReceiveFileForm = new frmReceiveFileForm();
           newReceiveFileForm.ShowDialog();
           newReceiveFileForm.Dispose();

       }

       private void btnClearBox_Click(object sender, EventArgs e)
       {
           textBox1.Clear();
       }

       private void clearScreenToolStripMenuItem_Click(object sender, EventArgs e)
       {
           textBox1.Clear();
       }

       private void hexConversionToolStripMenuItem_Click(object sender, EventArgs e)
       {
           PerformHexConversion();
       }
       
        private void btnHexConv_Click(object sender, EventArgs e)
       {
           PerformHexConversion();
       }

       private void PerformHexConversion()
       {
           HexConversion = !HexConversion;
           if (!HexConversion)
           {
               toolStripStatusLabel3.Text = "ASCII";
               IdxHexChar = 0;
           }
           else
               toolStripStatusLabel3.Text = "Hex Conversion";
       }

       
        //Timer1 tick maintains the global counter for Hyper Serial Port
       private void timer1_Tick(object sender, EventArgs e)
       {
           DisplayStopWatchElapsedTime(STOP_WATCH_TOOLSTRIP_LABEL);
       }

       public void DisplayStopWatchElapsedTime(int displayDevice)
       {
           TimeSpan ts = stopWatch.Elapsed;

           if (ts.Seconds > LastTimerTickSeconds) 
           {
               string elapsedTime = String.Format("{0:00}:{1:00}:{2:00}", ts.Hours, ts.Minutes, ts.Seconds);
               switch(displayDevice)
               {
                   case STOP_WATCH_TOOLSTRIP_LABEL:
                        toolStripStatusLabel1.Text = "Connected " + elapsedTime;
                        break;
                   case STOP_WATCH_TEXTBOX1:
                        textBox1.AppendText(" " + elapsedTime);
                        break;
               }
               if (ts.Seconds >= 59)
                   LastTimerTickSeconds = -1;
               else
                   LastTimerTickSeconds = ts.Seconds;
           }
       }

       private void mnuSendCharacters_Click(object sender, EventArgs e)
       {

           SendCharactersForm();
       }

       public void SendCharactersForm()
       {
           serialPort1.Close();
           //This instance allows communications with the SendCharactersForm.
           frmSendCharactersForm newSendCharactersForm = new frmSendCharactersForm();
           newSendCharactersForm.ShowDialog();
           newSendCharactersForm.Dispose();
           OpenSerialPort1();
           SendCharMicroTimerStartFlag = false;
           sendCharState = SEND_CHAR_OPEN_PORT;

       }

        /*
       public void SendCharTestStart()
       {
           //Open serial port1 at the start of the Send Character testing. The 
           //serial port will be closed at the exit of the Send Character testing.
           //This is done because a child window cannot control the serial port directly.
           OpenSerialPort1();
           //Set SendCharTestStartFlag to true to allow the timer to reset
           SendCharTestStartFlag = true;
           PutBuff = SendCharString.ToCharArray(0, SendCharString.Length);
           if (millisecondTimerFlag)
               sendchar_Timer = new System.Threading.Timer(Timer_Tick, null, SendTimerInterval, 0);
           else
               MicroTimerTest();
       }
         */
              public void SendCharTestStart()
              {
                  PutBuff = SendCharString.ToCharArray(0, SendCharString.Length);
                  //Open serial port1 at the start of the Send Character testing. The 
                  //serial port will be closed at the exit of the Send Character testing.
                  //This is done because a child window cannot control the serial port directly.
                  switch(sendCharState)
                 {
                      case SEND_CHAR_OPEN_PORT:
                            OpenSerialPort1();
                            if (SendCharRepetitive)
                            {
                                sendCharState = SEND_CHAR_RUN_TEST;
                                sendCharDisplayState = SEND_CHAR_DISPLAY_REPITITIVE;
                            }
                            else if (SendCharOneShot)
                            {
                                sendCharState = SEND_CHAR_ONE_SHOT;
                                sendCharDisplayState = SEND_CHAR_DISPLAY_ONE_SHOT;
                            }
                      break;
                      case SEND_CHAR_ONE_SHOT:
                      break;
                      case SEND_CHAR_RUN_TEST:
                      break;
                      default:
                      break;
                 }
                      
         
                  //Set SendCharTestStartFlag to true to allow the timer to reset
                  SendCharTestStartFlag = true;
                  if (millisecondTimerFlag)
                      sendchar_Timer = new System.Threading.Timer(Timer_Tick, null, SendTimerInterval, 0);
                  else
                      MicroTimerTest();
              }

       public void SendCharStopMillisecondTimer()
       {
           //Set SendCharTestStartFlag to false to Dispose of the timer
           SendCharTestStartFlag = false;
       }
       
        //Timer tick for the millisecond timer
       private void Timer_Tick(object state)
       {
           try
           {
               CallPutData();
           }
           finally
           {
               if (!SendCharTestStartFlag | SendCharOneShot)
               {
                   sendchar_Timer.Dispose();
                   serialPort1.Close();
               }
               else
               {
                   sendchar_Timer.Change(SendTimerInterval, 0);
                   if (SendCharIncrement7Bit)
                   {
                       if (PutBuff[0] < 0x7f)
                           PutBuff[0]++;
                       else
                           PutBuff[0] = '0';
                   }
                   else if (SendCharIncrementByte)
                       PutBuff[0]++;
               }
           }
    
       }

       //public delegate void SendCharStopTimerHandler(object o, SendCharStopTimerArgs e);

       //public class SendCharStopTimerArgs : EventArgs
       //{
       //    public bool SendCharStopTimerFlag;
       //
       //    public SendCharStopTimerArgs(bool flag)
       //    {
       //        SendCharStopTimerFlag = flag;
       //    }

       // }

       //public event SendCharStopTimerHandler SendCharStopTimerEvent;

       private void MicroTimerTest()
       {
           //MicroLibrary.MicroTimer microTimer = new MicroLibrary.MicroTimer();
           if (!SendCharMicroTimerStartFlag)
           {
               microTimer.MicroTimerElapsed += new MicroLibrary.MicroTimer.MicroTimerElapsedEventHandler(OnTimedEvent);
               //SendCharStopTimerEvent += new SendCharStopTimerHandler(SendCharStopTimerFunc);
               SendCharMicroTimerStartFlag = true;
           }
           //Set the sendCharDisplayState to the correct state
           if (SendCharOneShot)
           {
               sendCharDisplayState = SEND_CHAR_DISPLAY_ONE_SHOT;
               sendCharOneShotFire = false;
           }
           else
               sendCharDisplayState = SEND_CHAR_DISPLAY_REPITITIVE;
           // Call micro timer every 1000µs (1ms)
           microTimer.Interval = SendTimerInterval;
           // Can choose to ignore event if late by Xµs (by default it will try to catch up)  
           microTimer.IgnoreEventIfLateBy = 500;
           // Start timer
           microTimer.Enabled = true;
           // Do something whilst events happening, for demo sleep 4000ms (4sec)   
           //System.Threading.Thread.Sleep(4000); 
           // Stop timer
           //microTimer.Enabled = false;              


       }


       private void OnTimedEvent(object sender, MicroLibrary.MicroTimerEventArgs timerEventArgs)
       {
           switch(sendCharDisplayState) 
           {
               case SEND_CHAR_DISPLAY_REPITITIVE:
                   if(SendCharOneShot)
                       sendCharDisplayState = SEND_CHAR_DISPLAY_ONE_SHOT;
                   break;
               case SEND_CHAR_DISPLAY_ONE_SHOT:
                   if (!sendCharOneShotFire)
                   {
                       sendCharDisplayState = SEND_CHAR_DISPLAY_IDLE;
                       sendCharOneShotFire = true;
                   }
                   break;
               case SEND_CHAR_DISPLAY_IDLE:
                   return;
                   break;
               default:
                   return;
                   break;
           }

                   //Console.WriteLine(string.Format("Count = {0:#,0}  Timer = {1:#,0} µs, LateBy = {2:#,0} µs, ExecutionTime = {3:#,0} µs",
                   //    timerEventArgs.TimerCount, timerEventArgs.ElapsedMicroseconds, timerEventArgs.TimerLateBy, timerEventArgs.CallbackFunctionExecutionTime));
                   // Send the one character buffer.
                   this.SendThread = new Thread(new ThreadStart(this.CallPutData));
                   this.SendThread.Start();

                   //if (SendCharOneShot)
                   //{
                   //    SendCharStopTimerArgs eOneShotFlag = new SendCharStopTimerArgs(SendCharOneShot);
                   //    SendCharStopTimerEvent(new object(), eOneShotFlag);              // Stop timer
                   //    SendCharOneShot = false;
                   //}

                   if (SendCharIncrement7Bit)
                   {
                       if (PutBuff[0] < 0x7f)
                           PutBuff[0]++;
                       else
                           PutBuff[0] = ' ';
                   }
                   else if (SendCharIncrementByte)
                   {
                       if (PutBuff[0] < 0xff)
                           PutBuff[0]++;
                       else
                           PutBuff[0] = '\n';//'0';
                   }
       }

       //public void SendCharStopTimerFunc(object o, SendCharStopTimerArgs e)
       //{
       //    if(e.SendCharStopTimerFlag)
       //        MessageBox.Show("SendCharOneShot = " + e.SendCharStopTimerFlag);
       //        SendCharStopMicroTimer();              // Stop timer
       //}

       public void SendCharStopMicroTimer()
       {
           // Set the microTimer.Enable to false, clear the PutBuff.
           microTimer.Enabled = false;              // Stop timer
           System.Threading.Thread.Sleep(1000);
           Array.Clear(PutBuff, 0,PutBuff.Length);
           //microTimer.Dispose();
           return;
       }

       public void SendCharStopSerialPort1()
       {
           serialPort1.Close();
           return;
       }

        private void mnuExtTrigger_Click(object sender, EventArgs e)
       {
           ExtTriggerForm();
       }

       public void ExtTriggerForm()
       {
           //serialPort1.Close();
           //This instance allows a windows form that displays the external trigger functions.
           frmExtTriggerForm newExtTriggerForm = new frmExtTriggerForm();
           newExtTriggerForm.ShowDialog();
           newExtTriggerForm.Dispose();

       }


       private void mnuAboutHyperSerialPort_Click(object sender, EventArgs e)
       {
           frmAboutHyperSerialPortForm newAboutHyperSerialPortForm = new frmAboutHyperSerialPortForm();
           newAboutHyperSerialPortForm.ShowDialog();
           newAboutHyperSerialPortForm.Dispose();
       }

       private void mnuHelpTopics_Click(object sender, EventArgs e)
       {
           string helpFilePath = @"HSP_Help_File.chm";

           if (File.Exists(helpFilePath))
           {
               Process.Start(helpFilePath);
           }
           else
           {
               MessageBox.Show(string.Format("File not Found [{0}]", helpFilePath));
           }
       }

       private void mnuMultiTerminal_Click(object sender, EventArgs e)
       {
           mnuMultiTerminal_Open();
       }

       private void mnuMultiTerminal_Open()
       {
           //Create the windows form to set the MultiTerminal serial port properties
           frmMultiTerminalForm newMultiTerminalForm = new frmMultiTerminalForm();
           newMultiTerminalForm.ShowDialog();
           newMultiTerminalForm.Dispose();

           //Set up the new window and buttons for MultiTerminal
           if (mtStartMultiWindow)
           {
               //Try to open Multi Terminal serial port
               if( OpenSerialPortMultiTerminal())
                    //Open textbox for Multi Terminal
                    //Open Connect and Disconnect buttons
                    OpenMultiTerminalTextbox();
           }

       }

       private bool OpenSerialPortMultiTerminal()
       {
           try
           {
               serialPortMultiTerminal.PortName = mtPortName;
               serialPortMultiTerminal.BaudRate = Convert.ToInt32(mtBaudRate);
               serialPortMultiTerminal.Parity = (Parity)Enum.Parse(typeof(Parity), mtParity);
               serialPortMultiTerminal.DataBits = Convert.ToInt16(mtDataBits);
               serialPortMultiTerminal.StopBits = (StopBits)Enum.Parse(typeof(StopBits), mtStopBits);
               serialPortMultiTerminal.Handshake = (Handshake)Enum.Parse(typeof(Handshake), mtHandshake);
               if (!serialPortMultiTerminal.IsOpen)
               {
                   serialPortMultiTerminal.Open();
                   multiTerminalConnect.Enabled = false;
                   multiTerminalDisconnect.Enabled = true;
                    multiTerminalTextbox.ReadOnly = false;
                    toolStripStatusLabel5.Text = "Serial Port 2 " + mtPortName + "  " + mtBaudRate + " - " + mtDataBits + " - " + mtStopBits; ;
                    toolStripStatusLabel4.Text = "Serial Port 2 Connected";
               }
               return true;
           }
           catch (Exception ex)
           {
               MessageBox.Show(ex.Message);
               return false;
           }
       }

       private void CloseSerialPortMultiTerminal()
       {
               if (serialPortMultiTerminal.IsOpen)
                    serialPortMultiTerminal.Close();
               multiTerminalConnect.Enabled = true;
               multiTerminalDisconnect.Enabled = false;
               multiTerminalTextbox.ReadOnly = true;
               toolStripStatusLabel4.Text = "Serial Port 2 Disconnected";
       }

        public void OpenMultiTerminalTextbox()
       {
           multiTerminalTextbox = new TextBox();
           multiTerminalTextbox.Location = new System.Drawing.Point(575, 74);
           multiTerminalTextbox.Size = new System.Drawing.Size(500, 249);
           //multiTerminalTextbox.Text = "Multi Terminal Window";
           multiTerminalTextbox.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
           multiTerminalTextbox.Visible = true;
           multiTerminalTextbox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
           multiTerminalTextbox.Multiline = true;
           this.multiTerminalTextbox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.multiTerminalTextbox_KeyPress);
           this.Controls.Add(multiTerminalTextbox);
           //Set the anchor style for the multi window
           this.multiTerminalTextbox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left))));

            //Size the textbox1 to its original size
            this.textBox1.Size = new System.Drawing.Size(554, 249);
           //Set the anchor style of textbox1 to accomodate the multi window
           this.textBox1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left))));
           //Resize the window for both textboxes.
           //this.textBox1.AutoSize = false;
           this.AutoSize = true;

           multiTerminalConnect = new Button();
           multiTerminalConnect.Location = new Point(650, 25);
           multiTerminalConnect.Image = new Bitmap("Connection 005 40x40.png");
           multiTerminalConnect.FlatAppearance.BorderSize = 0;
           multiTerminalConnect.FlatAppearance.MouseOverBackColor = System.Drawing.Color.Transparent;
           multiTerminalConnect.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
           multiTerminalConnect.Size = new System.Drawing.Size(40, 40);
           //multiTerminalConnect.Text = "Connect";
           this.Controls.Add(multiTerminalConnect);
           multiTerminalConnect.Click += new EventHandler(multiTerminalConnect_Click);

           multiTerminalDisconnect = new Button();
           multiTerminalDisconnect.Location = new Point(700, 25);
           multiTerminalDisconnect.Image = new Bitmap("No access 40x40.png");
           multiTerminalDisconnect.FlatAppearance.BorderSize = 0;
           multiTerminalDisconnect.FlatAppearance.MouseOverBackColor = System.Drawing.Color.Transparent;
           multiTerminalDisconnect.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
           multiTerminalDisconnect.Size = new System.Drawing.Size(40, 40);
           //multiTerminalDisconnect.Text = "Disconnect";
           this.Controls.Add(multiTerminalDisconnect);
           multiTerminalDisconnect.Click += new EventHandler(multiTerminalDisconnect_Click);

           multiTerminalConfigure = new Button();
           multiTerminalConfigure.Location = new Point(750, 25);
           multiTerminalConfigure.Image = new Bitmap("Properties 002 40x40.png");
           multiTerminalConfigure.FlatAppearance.BorderSize = 0;
           multiTerminalConfigure.FlatAppearance.MouseOverBackColor = System.Drawing.Color.Transparent;
           multiTerminalConfigure.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
           multiTerminalConfigure.Size = new System.Drawing.Size(40, 40);
           //multiTerminalConfigure.Text = "Configure";
           this.Controls.Add(multiTerminalConfigure);
           multiTerminalConfigure.Click += new EventHandler(multiTerminalConfigure_Click);

           multiTerminalClose = new Button();
           multiTerminalClose.Location = new Point(850, 25);
           multiTerminalClose.Image = new Bitmap("Wrong 40x40.png");
           multiTerminalClose.FlatAppearance.BorderSize = 0;
           multiTerminalClose.FlatAppearance.MouseOverBackColor = System.Drawing.Color.Transparent;
           multiTerminalClose.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
           multiTerminalClose.Size = new System.Drawing.Size(40, 40);
           //multiTerminalClose.Text = "Close";
           this.Controls.Add(multiTerminalClose);
           multiTerminalClose.Click += new EventHandler(multiTerminalClose_Click);

           multiTerminalLabel = new Label();
           multiTerminalLabel.Location = new Point(550, 40);
           multiTerminalLabel.Font = new Font("Times New Roman", 12,FontStyle.Bold);
           multiTerminalLabel.Text = "Serial Port 2";
           this.Controls.Add(multiTerminalLabel);

           // 
           // toolStripStatusLabel4 Multi Window Connection Status
           // 
           toolStripStatusLabel4 = new ToolStripStatusLabel();
            this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {toolStripStatusLabel4 });
           toolStripStatusLabel4.AutoSize = true;
           toolStripStatusLabel4.BorderSides = ((System.Windows.Forms.ToolStripStatusLabelBorderSides)((((System.Windows.Forms.ToolStripStatusLabelBorderSides.Left | System.Windows.Forms.ToolStripStatusLabelBorderSides.Top)
                       | System.Windows.Forms.ToolStripStatusLabelBorderSides.Right)
                       | System.Windows.Forms.ToolStripStatusLabelBorderSides.Bottom)));
           toolStripStatusLabel4.BorderStyle = System.Windows.Forms.Border3DStyle.Bump;
           toolStripStatusLabel4.Name = "toolStripStatusLabel4";
           toolStripStatusLabel4.Size = new System.Drawing.Size(250, 19);
           toolStripStatusLabel4.Text = "toolStripStatusLabel4";

           // 
           // toolStripStatusLabel5 Multi Window Port Settings
           // 
           toolStripStatusLabel5 = new ToolStripStatusLabel();
           this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] { toolStripStatusLabel5 });
           toolStripStatusLabel5.AutoSize = true;
           toolStripStatusLabel5.BorderSides = ((System.Windows.Forms.ToolStripStatusLabelBorderSides)((((System.Windows.Forms.ToolStripStatusLabelBorderSides.Left | System.Windows.Forms.ToolStripStatusLabelBorderSides.Top)
                       | System.Windows.Forms.ToolStripStatusLabelBorderSides.Right)
                       | System.Windows.Forms.ToolStripStatusLabelBorderSides.Bottom)));
           toolStripStatusLabel5.BorderStyle = System.Windows.Forms.Border3DStyle.Bump;
           toolStripStatusLabel5.Name = "toolStripStatusLabel5";
           toolStripStatusLabel5.Size = new System.Drawing.Size(250, 19);
           toolStripStatusLabel5.Text = "toolStripStatusLabel5";
       }

       private void multiTerminalConnect_Click(object sender, EventArgs e)
       {
           OpenSerialPortMultiTerminal();
       }

       private void multiTerminalDisconnect_Click(object sender, EventArgs e)
       {
           if(serialPortMultiTerminal.IsOpen)
               CloseSerialPortMultiTerminal();
       }

       private void multiTerminalConfigure_Click(object sender, EventArgs e)
       {
           if (!serialPortMultiTerminal.IsOpen)
               mnuMultiTerminal_Open();
           else
               MessageBox.Show("Must Close Serial Port Before Accessing Properties");

       }


       private void multiTerminalClose_Click(object sender, EventArgs e)
       {
           mtStartMultiWindow = false;
           if (serialPortMultiTerminal.IsOpen)
               CloseSerialPortMultiTerminal();
           multiTerminalTextbox.Dispose();
           multiTerminalConnect.Dispose();
           multiTerminalDisconnect.Dispose();
           multiTerminalConfigure.Dispose();
           multiTerminalClose.Dispose();
           multiTerminalLabel.Dispose();
           toolStripStatusLabel4.Dispose();
           toolStripStatusLabel5.Dispose();
           //Resize the window for both textboxes.
           //this.AutoSize = true;
           //Reset the textbox1 anchor
           this.textBox1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                       | System.Windows.Forms.AnchorStyles.Left)
                       | System.Windows.Forms.AnchorStyles.Right)));
           //Resize the window for both textboxes.
           this.AutoSize = true;
           //this.Size = new System.Drawing.Size(600, 250);

       }
       
        private void multiTerminalTextbox_KeyPress(object sender, KeyPressEventArgs e)
       {

           // If the port is closed, don't try to send a character.
           if (!serialPortMultiTerminal.IsOpen) return;

           // If the port is Open, declare a char[] array with one element.
           PutBuffMultiWindow = new char[1];

           // Load element 0 with the key character.
           PutBuffMultiWindow[0] = e.KeyChar;

           // Send the one character buffer.
           this.SendThreadMultiWindow = new Thread(new ThreadStart(this.CallPutDataMultiWindow));
           this.SendThreadMultiWindow.Start();

           // Set the KeyPress event as handled so the character won't
           // display locally. If you want it to display, omit the next line.
           e.Handled = EchoTransmitCharacters;
       }


       private void serialPortMultiTerminal_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
       {
           //dataready = true;
           this.Invoke(new EventHandler(CallGetDataMultipleWindow));
       }



       //CallGetData is invoked to process the data in the serial port 1 buffer
       private void CallGetDataMultipleWindow(object sender, EventArgs e)
       {
           while (serialPortMultiTerminal.BytesToRead != 0)
           {
               //CallGetDataChar gets all characters in the serial port buffer.
               //Display thread is used to invoke a thread to display while not 
               //interrupting the reading of the serial port.
               CallGetDataCharMultiWindow = serialPortMultiTerminal.ReadExisting();
               this.DisplayThreadMultiWindow = new Thread(new ThreadStart(this.ThreadProcDisplayMultiWindow));
               this.DisplayThreadMultiWindow.Start();
           }
       }
       // This method is executed on the worker thread and makes
       // a thread-safe call on the TextBox control.
       private void ThreadProcDisplayMultiWindow()
       {
           //this.DisplayCharacters(Convert.ToChar(CallGetDataChar));
           this.DisplayCharactersMultiWindow(CallGetDataCharMultiWindow);

       }

       //DisplayCharacters() is thread safe method to display characters
       // from serial port on the main textbox
       private void DisplayCharactersMultiWindow(string text)
       {
           // InvokeRequired required compares the thread ID of the
           // calling thread to the thread ID of the creating thread.
           // If these threads are different, it returns true.
           if (this.multiTerminalTextbox.InvokeRequired)
           {
               DisplayCharactersCallbackMultiWindow d = new DisplayCharactersCallbackMultiWindow(DisplayCharactersMultiWindow);
               this.Invoke(d, new object[] { text });
           }
           else
           {
               if (HexConversion)
               {
                   //Perform a hex conversion on each char in the string and display it.
                   string HexChar;
                   foreach (char c in text)
                   {
                       int tmp = c;
                       HexChar = String.Format("{0:x2}", (uint)System.Convert.ToUInt32(tmp.ToString()));
                       if (c > 0x7f)
                           multiTerminalTextbox.AppendText("'Binary'" + " = 0x" + HexChar);
                       else if (c > 0x20)
                           multiTerminalTextbox.AppendText("'" + c + "'" + " = 0x" + HexChar);
                       else
                       {
                           ASCII_To_Hex.Conversion asciiDecode;
                           asciiDecode = new ASCII_To_Hex.Conversion();
                           multiTerminalTextbox.AppendText("'" + asciiDecode.HexMap(c) + "'" + " = 0x" + HexChar);
                           hexConversionTabFlag = true;
                       }
                       if (IdxHexChar++ < 4)
                       {
                           if (hexConversionTabFlag)
                           {
                               multiTerminalTextbox.AppendText("\t");
                               hexConversionTabFlag = false;
                           }
                           else
                               multiTerminalTextbox.AppendText("\t\t");
                       }
                       else
                       {
                           multiTerminalTextbox.AppendText("\r\n");
                           IdxHexChar = 0;
                       }
                   }
               }
               else
               {
                   //If HexConversion is false display each char in string.

                   if (AppendLineFeed)
                   {
                       foreach (char c in text)
                       {
                           switch (c)
                           {
                               case '\n':
                                   this.multiTerminalTextbox.AppendText("\r\n");
                                   break;
                               default:
                                   this.multiTerminalTextbox.AppendText(text);
                                   break;
                           }
                       }
                   }
                   else

                       this.multiTerminalTextbox.AppendText(text);
               }
           }
       }

       //CallPutData writes characters to the serial port
       //It can be used by thread safe calls.
       private void CallPutDataMultiWindow()
       {
           if (!AddLineFeedCharacter)
               serialPortMultiTerminal.Write(PutBuffMultiWindow, 0, 1);
           else
           {
               if (PutBuffMultiWindow[0] == '\r')
               {
                   serialPortMultiTerminal.Write(PutBuffMultiWindow, 0, 1);
                   Thread.Sleep(100);
                   PutBuffMultiWindow[0] = '\n';
                   serialPortMultiTerminal.Write(PutBuffMultiWindow, 0, 1);
               }
           }
       }

    }
}
